@@ -3,7 +3,7 @@
  *                    2000 Wim Taymans <wtay@chello.be>
  *                    2001 Bastien Nocera <hadess@hadess.net>
  *                    2003 Colin Walters <walters@verbum.org>
- *                    2005 Tim-Philipp M??ller <tim centricular net>
+ *                    2005 Tim-Philipp Müller <tim centricular net>
  *
  * gstgnomevfssink.c: 
  *
@@ -181,14 +181,15 @@ gst_gnome_vfs_sink_class_init (GstGnomeVFSSinkClass * klass)
 
   g_object_class_install_property (gobject_class, ARG_LOCATION,
       g_param_spec_string ("location", "File Location",
-          "Location of the file to write", NULL, G_PARAM_READWRITE));
+          "Location of the file to write", NULL,
+          G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
   g_object_class_install_property (gobject_class, ARG_URI,
       g_param_spec_boxed ("uri", "GnomeVFSURI", "URI for GnomeVFS",
-          GST_TYPE_GNOME_VFS_URI, G_PARAM_READWRITE));
+          GST_TYPE_GNOME_VFS_URI, G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
   g_object_class_install_property (gobject_class, ARG_HANDLE,
-      g_param_spec_boxed ("handle",
-          "GnomeVFSHandle", "Handle for GnomeVFS",
-          GST_TYPE_GNOME_VFS_HANDLE, G_PARAM_READWRITE));
+      g_param_spec_boxed ("handle", "GnomeVFSHandle", "Handle for GnomeVFS",
+          GST_TYPE_GNOME_VFS_HANDLE,
+          G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
 
   /**
    * GstGnomeVFSSink::allow-overwrite
@@ -242,7 +243,7 @@ gst_gnome_vfs_sink_init (GstGnomeVFSSink * sink, GstGnomeVFSSinkClass * klass)
   sink->uri_name = NULL;
   sink->handle = NULL;
   sink->own_handle = FALSE;
-  sink->data_written = 0;
+  sink->current_pos = 0;
 
   GST_BASE_SINK (sink)->sync = FALSE;
 }
@@ -398,7 +399,7 @@ gst_gnome_vfs_sink_open_file (GstGnomeVFSSink * sink)
     sink->own_handle = FALSE;
   }
 
-  sink->data_written = 0;
+  sink->current_pos = 0;
 
   return TRUE;
 }
@@ -448,6 +449,7 @@ static gboolean
 gst_gnome_vfs_sink_handle_event (GstBaseSink * basesink, GstEvent * event)
 {
   GstGnomeVFSSink *sink;
+  gboolean ret = TRUE;
 
   sink = GST_GNOME_VFS_SINK (basesink);
 
@@ -474,6 +476,9 @@ gst_gnome_vfs_sink_handle_event (GstBaseSink * basesink, GstEvent * event)
       if (res != GNOME_VFS_OK) {
         GST_ERROR_OBJECT (sink, "Failed to seek to offset %"
             G_GINT64_FORMAT ": %s", offset, gnome_vfs_result_to_string (res));
+        ret = FALSE;
+      } else {
+        sink->current_pos = offset;
       }
 
       break;
@@ -481,14 +486,14 @@ gst_gnome_vfs_sink_handle_event (GstBaseSink * basesink, GstEvent * event)
 
     case GST_EVENT_FLUSH_START:
     case GST_EVENT_EOS:{
-      /* how does Gnome-VFS flush? Do we need to flush? */
+      /* No need to flush with GnomeVfs */
       break;
     }
     default:
       break;
   }
 
-  return TRUE;
+  return ret;
 }
 
 static gboolean
@@ -505,7 +510,7 @@ gst_gnome_vfs_sink_query (GstPad * pad, GstQuery * query)
       switch (format) {
         case GST_FORMAT_DEFAULT:
         case GST_FORMAT_BYTES:
-          gst_query_set_position (query, GST_FORMAT_BYTES, sink->data_written);
+          gst_query_set_position (query, GST_FORMAT_BYTES, sink->current_pos);
           return TRUE;
         default:
           return FALSE;
@@ -527,13 +532,12 @@ gst_gnome_vfs_sink_render (GstBaseSink * basesink, GstBuffer * buf)
   GstGnomeVFSSink *sink;
   GnomeVFSResult result;
   GstFlowReturn ret;
-  guint64 back_pending = 0;
 
   sink = GST_GNOME_VFS_SINK (basesink);
 
   if (gnome_vfs_tell (sink->handle, &cur_pos) == GNOME_VFS_OK) {
-    if (cur_pos < sink->data_written)
-      back_pending = sink->data_written - cur_pos;
+    /* bring up to date with current position for proper reporting */
+    sink->current_pos = cur_pos;
   }
 
   result = gnome_vfs_write (sink->handle, GST_BUFFER_DATA (buf),
@@ -551,7 +555,7 @@ gst_gnome_vfs_sink_render (GstBaseSink * basesink, GstBuffer * buf)
             GST_BUFFER_SIZE (buf), written);
       }
 
-      sink->data_written += GST_BUFFER_SIZE (buf) - back_pending;
+      sink->current_pos += GST_BUFFER_SIZE (buf);
       ret = GST_FLOW_OK;
       break;
     }
@@ -592,12 +596,7 @@ gst_gnome_vfs_sink_uri_get_type (void)
 static gchar **
 gst_gnome_vfs_sink_uri_get_protocols (void)
 {
-  static gchar **protocols = NULL;
-
-  if (!protocols)
-    protocols = gst_gnomevfs_get_supported_uris ();
-
-  return protocols;
+  return gst_gnomevfs_get_supported_uris ();
 }
 
 static const gchar *
