@@ -69,6 +69,15 @@
 #define GST_CAT_DEFAULT theoraenc_debug
 GST_DEBUG_CATEGORY_STATIC (GST_CAT_DEFAULT);
 
+/* With libtheora-1.0beta1 the granulepos scheme was changed:
+ * where earlier the granulepos refered to the index/beginning
+ * of a frame, it now refers to the end, which matches the use
+ * in vorbis/speex. There don't seem to be defines for the
+ * theora version we're compiling against, so we'll just use
+ * a run-time check for now. See theora_enc_get_ogg_packet_end_time().
+ */
+static gboolean use_old_granulepos;
+
 #define GST_TYPE_BORDER_MODE (gst_border_mode_get_type())
 static GType
 gst_border_mode_get_type (void)
@@ -211,53 +220,59 @@ gst_theora_enc_class_init (GstTheoraEncClass * klass)
   g_object_class_install_property (gobject_class, ARG_CENTER,
       g_param_spec_boolean ("center", "Center",
           "Center image when sizes not multiple of 16", THEORA_DEF_CENTER,
-          (GParamFlags) G_PARAM_READWRITE));
+          (GParamFlags) G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
   g_object_class_install_property (gobject_class, ARG_BORDER,
       g_param_spec_enum ("border", "Border",
           "Border color to add when sizes not multiple of 16",
           GST_TYPE_BORDER_MODE, THEORA_DEF_BORDER,
-          (GParamFlags) G_PARAM_READWRITE));
+          (GParamFlags) G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
   /* general encoding stream options */
   g_object_class_install_property (gobject_class, ARG_BITRATE,
       g_param_spec_int ("bitrate", "Bitrate", "Compressed video bitrate (kbps)",
-          0, 2000, THEORA_DEF_BITRATE, (GParamFlags) G_PARAM_READWRITE));
+          0, 2000, THEORA_DEF_BITRATE,
+          (GParamFlags) G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
   g_object_class_install_property (gobject_class, ARG_QUALITY,
-      g_param_spec_int ("quality", "Quality", "Video quality",
-          0, 63, THEORA_DEF_QUALITY, (GParamFlags) G_PARAM_READWRITE));
+      g_param_spec_int ("quality", "Quality", "Video quality", 0, 63,
+          THEORA_DEF_QUALITY,
+          (GParamFlags) G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
   g_object_class_install_property (gobject_class, ARG_QUICK,
       g_param_spec_boolean ("quick", "Quick", "Quick encoding",
-          THEORA_DEF_QUICK, (GParamFlags) G_PARAM_READWRITE));
+          THEORA_DEF_QUICK,
+          (GParamFlags) G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
   g_object_class_install_property (gobject_class, ARG_KEYFRAME_AUTO,
       g_param_spec_boolean ("keyframe-auto", "Keyframe Auto",
           "Automatic keyframe detection", THEORA_DEF_KEYFRAME_AUTO,
-          (GParamFlags) G_PARAM_READWRITE));
+          (GParamFlags) G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
   g_object_class_install_property (gobject_class, ARG_KEYFRAME_FREQ,
       g_param_spec_int ("keyframe-freq", "Keyframe frequency",
           "Keyframe frequency", 1, 32768, THEORA_DEF_KEYFRAME_FREQ,
-          (GParamFlags) G_PARAM_READWRITE));
+          (GParamFlags) G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
   g_object_class_install_property (gobject_class, ARG_KEYFRAME_FREQ_FORCE,
       g_param_spec_int ("keyframe-force", "Keyframe force",
           "Force keyframe every N frames", 1, 32768,
-          THEORA_DEF_KEYFRAME_FREQ_FORCE, (GParamFlags) G_PARAM_READWRITE));
+          THEORA_DEF_KEYFRAME_FREQ_FORCE,
+          (GParamFlags) G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
   g_object_class_install_property (gobject_class, ARG_KEYFRAME_THRESHOLD,
       g_param_spec_int ("keyframe-threshold", "Keyframe threshold",
           "Keyframe threshold", 0, 32768, THEORA_DEF_KEYFRAME_THRESHOLD,
-          (GParamFlags) G_PARAM_READWRITE));
+          (GParamFlags) G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
   g_object_class_install_property (gobject_class, ARG_KEYFRAME_MINDISTANCE,
       g_param_spec_int ("keyframe-mindistance", "Keyframe mindistance",
           "Keyframe mindistance", 1, 32768, THEORA_DEF_KEYFRAME_MINDISTANCE,
-          (GParamFlags) G_PARAM_READWRITE));
+          (GParamFlags) G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
   g_object_class_install_property (gobject_class, ARG_NOISE_SENSITIVITY,
       g_param_spec_int ("noise-sensitivity", "Noise sensitivity",
           "Noise sensitivity", 0, 32768, THEORA_DEF_NOISE_SENSITIVITY,
-          (GParamFlags) G_PARAM_READWRITE));
+          (GParamFlags) G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
   g_object_class_install_property (gobject_class, ARG_SHARPNESS,
-      g_param_spec_int ("sharpness", "Sharpness",
-          "Sharpness", 0, 2, THEORA_DEF_SHARPNESS,
-          (GParamFlags) G_PARAM_READWRITE));
+      g_param_spec_int ("sharpness", "Sharpness", "Sharpness", 0, 2,
+          THEORA_DEF_SHARPNESS,
+          (GParamFlags) G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
 
   gstelement_class->change_state = theora_enc_change_state;
   GST_DEBUG_CATEGORY_INIT (theoraenc_debug, "theoraenc", 0, "Theora encoder");
+
+  use_old_granulepos = (theora_version_number () <= 0x00030200);
 }
 
 static void
@@ -540,6 +555,25 @@ theora_set_header_on_caps (GstCaps * caps, GstBuffer * buf1,
   return caps;
 }
 
+static GstClockTime
+theora_enc_get_ogg_packet_end_time (GstTheoraEnc * enc, ogg_packet * op)
+{
+  ogg_int64_t end_granule;
+
+  /* FIXME: remove this hack once we depend on libtheora >= 1.0beta1 */
+  if (G_UNLIKELY (use_old_granulepos)) {
+    /* This is where we hack around theora's broken idea of what granulepos
+     * is -- normally we wouldn't need to add the 1, because granulepos
+     * should be the presentation time of the last sample in the packet, but
+     * theora starts with 0 instead of 1... (update: this only applies to old
+     * bitstream/theora versions, this is fixed with bitstream version 3.2.1) */
+    end_granule = granulepos_add (op->granulepos, 1, enc->granule_shift);
+  } else {
+    end_granule = op->granulepos;
+  }
+  return theora_granule_time (&enc->state, end_granule) * GST_SECOND;
+}
+
 static gboolean
 theora_enc_sink_event (GstPad * pad, GstEvent * event)
 {
@@ -554,10 +588,8 @@ theora_enc_sink_event (GstPad * pad, GstEvent * event)
       if (enc->initialised) {
         /* push last packet with eos flag */
         while (theora_encode_packetout (&enc->state, 1, &op)) {
-          /* See comment in the chain function */
-          GstClockTime next_time = theora_granule_time (&enc->state,
-              granulepos_add (op.granulepos, 1, enc->granule_shift)) *
-              GST_SECOND;
+          GstClockTime next_time =
+              theora_enc_get_ogg_packet_end_time (enc, &op);
 
           theora_push_packet (enc, &op, enc->next_ts, next_time - enc->next_ts);
           enc->next_ts = next_time;
@@ -574,7 +606,7 @@ theora_enc_sink_event (GstPad * pad, GstEvent * event)
       if (gst_structure_has_name (s, "GstForceKeyUnit")) {
         GstClockTime next_ts;
 
-        /* make sure timestamps increment after reseting the decoder */
+        /* make sure timestamps increment after resetting the decoder */
         next_ts = enc->next_ts + enc->timestamp_offset;
 
         theora_enc_reset (enc);
@@ -887,14 +919,8 @@ theora_enc_chain (GstPad * pad, GstBuffer * buffer)
 
     ret = GST_FLOW_OK;
     while (theora_encode_packetout (&enc->state, 0, &op)) {
-      /* This is where we hack around theora's broken idea of what granulepos
-         is -- normally we wouldn't need to add the 1, because granulepos
-         should be the presentation time of the last sample in the packet, but
-         theora starts with 0 instead of 1... */
-      GstClockTime next_time;
+      GstClockTime next_time = theora_enc_get_ogg_packet_end_time (enc, &op);
 
-      next_time = theora_granule_time (&enc->state,
-          granulepos_add (op.granulepos, 1, enc->granule_shift)) * GST_SECOND;
       ret =
           theora_push_packet (enc, &op, enc->next_ts, next_time - enc->next_ts);
       enc->next_ts = next_time;
