@@ -54,8 +54,10 @@ static const GstTagEntryMatch tag_matches[] = {
   {GST_TAG_ALBUM_VOLUME_COUNT, "DISCTOTAL"},
   {GST_TAG_ARTIST, "ARTIST"},
   {GST_TAG_PERFORMER, "PERFORMER"},
+  {GST_TAG_COMPOSER, "COMPOSER"},
   {GST_TAG_COPYRIGHT, "COPYRIGHT"},
   {GST_TAG_LICENSE, "LICENSE"},
+  {GST_TAG_LICENSE_URI, "LICENSE"},
   {GST_TAG_LOCATION, "LOCATION"},
   {GST_TAG_ORGANIZATION, "ORGANIZATION"},
   {GST_TAG_DESCRIPTION, "DESCRIPTION"},
@@ -74,11 +76,20 @@ static const GstTagEntryMatch tag_matches[] = {
   {GST_TAG_MUSICBRAINZ_ALBUMID, "MUSICBRAINZ_ALBUMID"},
   {GST_TAG_MUSICBRAINZ_ALBUMARTISTID, "MUSICBRAINZ_ALBUMARTISTID"},
   {GST_TAG_MUSICBRAINZ_TRMID, "MUSICBRAINZ_TRMID"},
-  {GST_TAG_MUSICBRAINZ_SORTNAME, "MUSICBRAINZ_SORTNAME"},
+  {GST_TAG_ARTIST_SORTNAME, "ARTISTSORT"},
+  {GST_TAG_ARTIST_SORTNAME, "ARTISTSORTORDER"},
+  {GST_TAG_ARTIST_SORTNAME, "MUSICBRAINZ_SORTNAME"},
+  {GST_TAG_ALBUM_SORTNAME, "ALBUMSORT"},
+  {GST_TAG_ALBUM_SORTNAME, "ALBUMSORTORDER"},
+  {GST_TAG_TITLE_SORTNAME, "TITLESORT"},
+  {GST_TAG_TITLE_SORTNAME, "TITLESORTORDER"},
   {GST_TAG_LANGUAGE_CODE, "LANGUAGE"},
   {GST_TAG_CDDA_MUSICBRAINZ_DISCID, "MUSICBRAINZ_DISCID"},
   {GST_TAG_CDDA_CDDB_DISCID, "DISCID"},
-  /* some incidence that this makes sense:
+  /* For the apparent de-facto standard for coverart in vorbis comments, see:
+   * http://www.hydrogenaudio.org/forums/lofiversion/index.php/t48386.html */
+  {GST_TAG_PREVIEW_IMAGE, "COVERART"},
+  /* some evidence that "BPM" is used elsewhere:
    * http://mail.kde.org/pipermail/amarok/2006-May/000090.html
    */
   {GST_TAG_BEATS_PER_MINUTE, "BPM"},
@@ -225,6 +236,12 @@ gst_vorbis_tag_add (GstTagList * list, const gchar * tag, const gchar * value)
         } else if (strlen (value) != 2 && strlen (value) != 3) {
           GST_WARNING ("doesn't contain an ISO-639 language code: %s", value);
         }
+      } else if (strcmp (tag, "LICENSE") == 0) {
+        /* license tags in vorbis comments must contain an URI representing
+         * the license and nothing more, at least according to:
+         * http://wiki.xiph.org/index.php/LICENSE_and_COPYRIGHT_tags_on_Vorbis_Comments */
+        if (value && gst_uri_is_valid (value))
+          gst_tag = GST_TAG_LICENSE_URI;
       }
 
       if (!valid) {
@@ -287,6 +304,68 @@ gst_vorbis_tag_add (GstTagList * list, const gchar * tag, const gchar * value)
   }
 }
 
+static void
+gst_vorbis_tag_add_coverart (GstTagList * tags, const gchar * img_data_base64,
+    gint base64_len)
+{
+  GstBuffer *img;
+  guchar *img_data;
+  gsize img_len;
+  guint save = 0;
+  gint state = 0;
+
+  if (base64_len < 2)
+    goto not_enough_data;
+
+  img_data = g_try_malloc0 (base64_len * 3 / 4);
+
+  if (img_data == NULL)
+    goto alloc_failed;
+
+  img_len = g_base64_decode_step (img_data_base64, base64_len, img_data,
+      &state, &save);
+
+  if (img_len == 0)
+    goto decode_failed;
+
+  img = gst_tag_image_data_to_image_buffer (img_data, img_len,
+      GST_TAG_IMAGE_TYPE_NONE);
+
+  if (img == NULL)
+    goto convert_failed;
+
+  gst_tag_list_add (tags, GST_TAG_MERGE_APPEND,
+      GST_TAG_PREVIEW_IMAGE, img, NULL);
+
+  gst_buffer_unref (img);
+  g_free (img_data);
+  return;
+
+/* ERRORS */
+not_enough_data:
+  {
+    GST_WARNING ("COVERART tag with too little base64-encoded data");
+    return;
+  }
+alloc_failed:
+  {
+    GST_WARNING ("Couldn't allocate enough memory to decode COVERART tag");
+    return;
+  }
+decode_failed:
+  {
+    GST_WARNING ("Couldn't decode bas64 image data from COVERART tag");
+    g_free (img_data);
+    return;
+  }
+convert_failed:
+  {
+    GST_WARNING ("Couldn't extract image or image type from COVERART tag");
+    g_free (img_data);
+    return;
+  }
+}
+
 /**
  * gst_tag_list_from_vorbiscomment_buffer:
  * @buffer: buffer to convert
@@ -319,7 +398,7 @@ gst_tag_list_from_vorbiscomment_buffer (const GstBuffer * buffer,
   guint cur_size;
   guint iterations;
   guint8 *data;
-  guint size;
+  guint size, value_len;
   GstTagList *list;
 
   g_return_val_if_fail (GST_IS_BUFFER (buffer), NULL);
@@ -331,14 +410,19 @@ gst_tag_list_from_vorbiscomment_buffer (const GstBuffer * buffer,
 
   if (size < 11 || size <= id_data_length + 4)
     goto error;
+
   if (id_data_length > 0 && memcmp (data, id_data, id_data_length) != 0)
     goto error;
+
   ADVANCE (id_data_length);
+
   if (vendor_string)
     *vendor_string = g_strndup (cur, cur_size);
+
   ADVANCE (cur_size);
   iterations = cur_size;
   cur_size = 0;
+
   while (iterations) {
     ADVANCE (cur_size);
     iterations--;
@@ -350,11 +434,19 @@ gst_tag_list_from_vorbiscomment_buffer (const GstBuffer * buffer,
     }
     *value = '\0';
     value++;
-    if (!g_utf8_validate (value, -1, NULL)) {
+    value_len = strlen (value);
+    if (value_len == 0 || !g_utf8_validate (value, value_len, NULL)) {
       g_free (cur);
       continue;
     }
-    gst_vorbis_tag_add (list, cur, value);
+    /* we'll just ignore COVERARTMIME and typefind the image data */
+    if (g_ascii_strcasecmp (cur, "COVERARTMIME") == 0) {
+      continue;
+    } else if (g_ascii_strcasecmp (cur, "COVERART") == 0) {
+      gst_vorbis_tag_add_coverart (list, value, value_len);
+    } else {
+      gst_vorbis_tag_add (list, cur, value);
+    }
     g_free (cur);
   }
 
@@ -400,6 +492,8 @@ gst_tag_to_vorbis_comments (const GstTagList * list, const gchar * tag)
       return NULL;
   }
 
+  /* FIXME: for tags that can map to multiple vorbis comment keys, add all
+   * of the possible keys */
   for (i = 0; i < gst_tag_list_get_tag_size (list, tag); i++) {
     GType tag_type = gst_tag_get_type (tag);
     gchar *result = NULL;
