@@ -167,6 +167,12 @@ gst_ffmpeg_pixfmt_to_caps (enum PixelFormat pix_fmt, AVCodecContext * context)
     case PIX_FMT_YUV420P:
       fmt = GST_MAKE_FOURCC ('I', '4', '2', '0');
       break;
+    case PIX_FMT_NV12:
+      fmt = GST_MAKE_FOURCC ('N', 'V', '1', '2');
+      break;
+    case PIX_FMT_NV21:
+      fmt = GST_MAKE_FOURCC ('N', 'V', '2', '1');
+      break;
     case PIX_FMT_YVU420P:
       fmt = GST_MAKE_FOURCC ('Y', 'V', '1', '2');
       break;
@@ -176,6 +182,9 @@ gst_ffmpeg_pixfmt_to_caps (enum PixelFormat pix_fmt, AVCodecContext * context)
     case PIX_FMT_UYVY422:
       fmt = GST_MAKE_FOURCC ('U', 'Y', 'V', 'Y');
       break;
+    case PIX_FMT_UYVY411:
+      fmt = GST_MAKE_FOURCC ('I', 'Y', 'U', '1');
+      break;
     case PIX_FMT_RGB24:
       bpp = depth = 24;
       endianness = G_BIG_ENDIAN;
@@ -222,6 +231,34 @@ gst_ffmpeg_pixfmt_to_caps (enum PixelFormat pix_fmt, AVCodecContext * context)
       r_mask = 0x00ff0000;
       g_mask = 0x0000ff00;
       b_mask = 0x000000ff;
+#endif
+      break;
+    case PIX_FMT_xRGB32:
+      bpp = 32;
+      depth = 24;
+      endianness = G_BIG_ENDIAN;
+#if (G_BYTE_ORDER == G_BIG_ENDIAN)
+      r_mask = 0xff000000;
+      g_mask = 0x00ff0000;
+      b_mask = 0x0000ff00;
+#else
+      r_mask = 0x000000ff;
+      g_mask = 0x0000ff00;
+      b_mask = 0x00ff0000;
+#endif
+      break;
+    case PIX_FMT_BGRx32:
+      bpp = 32;
+      depth = 24;
+      endianness = G_BIG_ENDIAN;
+#if (G_BYTE_ORDER == G_BIG_ENDIAN)
+      r_mask = 0x000000ff;
+      g_mask = 0x0000ff00;
+      b_mask = 0x00ff0000;
+#else
+      r_mask = 0xff000000;
+      g_mask = 0x00ff0000;
+      b_mask = 0x0000ff00;
 #endif
       break;
     case PIX_FMT_RGBA32:
@@ -254,6 +291,38 @@ gst_ffmpeg_pixfmt_to_caps (enum PixelFormat pix_fmt, AVCodecContext * context)
       g_mask = 0x0000ff00;
       b_mask = 0x000000ff;
       a_mask = 0xff000000;
+#endif
+      break;
+    case PIX_FMT_ARGB32:
+      bpp = 32;
+      depth = 32;
+      endianness = G_BIG_ENDIAN;
+#if (G_BYTE_ORDER == G_BIG_ENDIAN)
+      r_mask = 0xff000000;
+      g_mask = 0x00ff0000;
+      b_mask = 0x0000ff00;
+      a_mask = 0x000000ff;
+#else
+      r_mask = 0x000000ff;
+      g_mask = 0x0000ff00;
+      b_mask = 0x00ff0000;
+      a_mask = 0xff000000;
+#endif
+      break;
+    case PIX_FMT_ABGR32:
+      bpp = 32;
+      depth = 32;
+      endianness = G_BIG_ENDIAN;
+#if (G_BYTE_ORDER == G_BIG_ENDIAN)
+      r_mask = 0x000000ff;
+      g_mask = 0x0000ff00;
+      b_mask = 0x00ff0000;
+      a_mask = 0xff000000;
+#else
+      r_mask = 0xff000000;
+      g_mask = 0x00ff0000;
+      b_mask = 0x0000ff00;
+      a_mask = 0x000000ff;
 #endif
       break;
     case PIX_FMT_YUV410P:
@@ -472,7 +541,7 @@ gst_ffmpeg_caps_to_smpfmt (const GstCaps * caps,
 
   if (gst_structure_get_int (structure, "width", &width) &&
       gst_structure_get_int (structure, "depth", &depth) &&
-      gst_structure_get_int (structure, "signed", &signedness) &&
+      gst_structure_get_boolean (structure, "signed", &signedness) &&
       gst_structure_get_int (structure, "endianness", &endianness)) {
     if (width == 16 && depth == 16 &&
         endianness == G_BYTE_ORDER && signedness == TRUE) {
@@ -524,9 +593,18 @@ gst_ffmpeg_caps_to_pixfmt (const GstCaps * caps,
         case GST_MAKE_FOURCC ('U', 'Y', 'V', 'Y'):
           context->pix_fmt = PIX_FMT_UYVY422;
           break;
+        case GST_MAKE_FOURCC ('I', 'Y', 'U', '1'):
+          context->pix_fmt = PIX_FMT_UYVY411;
+          break;
         case GST_MAKE_FOURCC ('I', '4', '2', '0'):
           context->pix_fmt = PIX_FMT_YUV420P;
           break;
+        case GST_MAKE_FOURCC ('N', 'V', '1', '2'):
+          context->pix_fmt = PIX_FMT_NV12;
+          break;
+        case GST_MAKE_FOURCC ('N', 'V', '2', '1'):
+          context->pix_fmt = PIX_FMT_NV21;
+          break;
         case GST_MAKE_FOURCC ('Y', 'V', '1', '2'):
           context->pix_fmt = PIX_FMT_YVU420P;
           break;
@@ -561,26 +639,43 @@ gst_ffmpeg_caps_to_pixfmt (const GstCaps * caps,
             if (gst_structure_get_int (structure, "alpha_mask", &amask)) {
 #if (G_BYTE_ORDER == G_BIG_ENDIAN)
               if (rmask == 0x0000ff00)
+                context->pix_fmt = PIX_FMT_BGRA32;
+              else if (rmask == 0x00ff0000)
+                context->pix_fmt = PIX_FMT_RGBA32;
+              else if (rmask == 0xff000000)
+                context->pix_fmt = PIX_FMT_ARGB32;
+              else              // if (r_mask = 0x000000ff)
+                context->pix_fmt = PIX_FMT_ABGR32;
 #else
               if (rmask == 0x00ff0000)
-#endif
                 context->pix_fmt = PIX_FMT_BGRA32;
-              else
+              else if (rmask == 0x0000ff00)
                 context->pix_fmt = PIX_FMT_RGBA32;
+              else if (rmask == 0x000000ff)
+                context->pix_fmt = PIX_FMT_ARGB32;
+              else              // if (rmask == 0xff000000)
+                context->pix_fmt = PIX_FMT_ABGR32;
+#endif
             } else {
 #if (G_BYTE_ORDER == G_BIG_ENDIAN)
               if (rmask == 0x00ff0000)
-#else
-              if (rmask == 0x0000ff00)
-#endif
                 context->pix_fmt = PIX_FMT_RGB32;
-
-#if (G_BYTE_ORDER == G_BIG_ENDIAN)
-              if (rmask == 0x0000ff00)
-#else
-              if (rmask == 0x00ff0000)
-#endif
+              else if (rmask == 0x0000ff00)
                 context->pix_fmt = PIX_FMT_BGR32;
+              else if (rmask == 0xff000000)
+                context->pix_fmt = PIX_FMT_xRGB32;
+              else              // if (rmask == 0x000000ff)
+                context->pix_fmt = PIX_FMT_BGRx32;
+#else
+              if (rmask == 0x0000ff00)
+                context->pix_fmt = PIX_FMT_RGB32;
+              else if (rmask == 0x00ff0000)
+                context->pix_fmt = PIX_FMT_BGR32;
+              else if (rmask == 0x000000ff)
+                context->pix_fmt = PIX_FMT_xRGB32;
+              else              // if (rmask == 0xff000000)
+                context->pix_fmt = PIX_FMT_BGRx32;
+#endif
             }
             break;
           case 24:
@@ -716,6 +811,22 @@ gst_ffmpegcsp_avpicture_fill (AVPicture * picture,
       picture->linesize[1] = stride2;
       picture->linesize[2] = stride2;
       return size + 2 * size2;
+    case PIX_FMT_NV12:
+    case PIX_FMT_NV21:
+      stride = GST_ROUND_UP_4 (width);
+      h2 = ROUND_UP_X (height, pinfo->y_chroma_shift);
+      size = stride * h2;
+      w2 = 2 * DIV_ROUND_UP_X (width, pinfo->x_chroma_shift);
+      stride2 = GST_ROUND_UP_4 (w2);
+      h2 = DIV_ROUND_UP_X (height, pinfo->y_chroma_shift);
+      size2 = stride2 * h2;
+      picture->data[0] = ptr;
+      picture->data[1] = picture->data[0] + size;
+      picture->data[2] = NULL;
+      picture->linesize[0] = stride;
+      picture->linesize[1] = stride2;
+      picture->linesize[2] = 0;
+      return size + size2;
     case PIX_FMT_RGB24:
     case PIX_FMT_BGR24:
       stride = GST_ROUND_UP_4 (width * 3);
@@ -728,8 +839,12 @@ gst_ffmpegcsp_avpicture_fill (AVPicture * picture,
     case PIX_FMT_AYUV4444:
     case PIX_FMT_RGB32:
     case PIX_FMT_RGBA32:
+    case PIX_FMT_ARGB32:
     case PIX_FMT_BGR32:
     case PIX_FMT_BGRA32:
+    case PIX_FMT_ABGR32:
+    case PIX_FMT_xRGB32:
+    case PIX_FMT_BGRx32:
       stride = width * 4;
       size = stride * height;
       picture->data[0] = ptr;
