@@ -86,9 +86,11 @@ GST_BOILERPLATE_FULL (GstGDPPay, gst_gdp_pay, GstElement,
     GST_TYPE_ELEMENT, _do_init);
 
 static void gst_gdp_pay_reset (GstGDPPay * this);
+
 static GstFlowReturn gst_gdp_pay_chain (GstPad * pad, GstBuffer * buffer);
 
 static gboolean gst_gdp_pay_src_event (GstPad * pad, GstEvent * event);
+
 static gboolean gst_gdp_pay_sink_event (GstPad * pad, GstEvent * event);
 
 static GstStateChangeReturn gst_gdp_pay_change_state (GstElement *
@@ -118,6 +120,7 @@ static void
 gst_gdp_pay_class_init (GstGDPPayClass * klass)
 {
   GObjectClass *gobject_class;
+
   GstElementClass *gstelement_class;
 
   gobject_class = (GObjectClass *) klass;
@@ -130,15 +133,16 @@ gst_gdp_pay_class_init (GstGDPPayClass * klass)
   g_object_class_install_property (gobject_class, PROP_CRC_HEADER,
       g_param_spec_boolean ("crc-header", "CRC Header",
           "Calculate and store a CRC checksum on the header",
-          DEFAULT_CRC_HEADER, G_PARAM_READWRITE));
+          DEFAULT_CRC_HEADER, G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
   g_object_class_install_property (gobject_class, PROP_CRC_PAYLOAD,
       g_param_spec_boolean ("crc-payload", "CRC Payload",
           "Calculate and store a CRC checksum on the payload",
-          DEFAULT_CRC_PAYLOAD, G_PARAM_READWRITE));
+          DEFAULT_CRC_PAYLOAD, G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
   g_object_class_install_property (gobject_class, PROP_VERSION,
       g_param_spec_enum ("version", "Version",
           "Version of the GStreamer Data Protocol",
-          GST_TYPE_DP_VERSION, DEFAULT_VERSION, G_PARAM_READWRITE));
+          GST_TYPE_DP_VERSION, DEFAULT_VERSION,
+          G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
 
   gstelement_class->change_state = GST_DEBUG_FUNCPTR (gst_gdp_pay_change_state);
 }
@@ -227,8 +231,11 @@ static GstBuffer *
 gst_gdp_buffer_from_caps (GstGDPPay * this, GstCaps * caps)
 {
   GstBuffer *headerbuf;
+
   GstBuffer *payloadbuf;
+
   guint8 *header, *payload;
+
   guint len;
 
   if (!this->packetizer->packet_from_caps (caps, this->header_flag, &len,
@@ -259,7 +266,9 @@ static GstBuffer *
 gst_gdp_pay_buffer_from_buffer (GstGDPPay * this, GstBuffer * buffer)
 {
   GstBuffer *headerbuf;
+
   guint8 *header;
+
   guint len;
 
   if (!this->packetizer->header_from_buffer (buffer, this->header_flag, &len,
@@ -288,9 +297,13 @@ static GstBuffer *
 gst_gdp_buffer_from_event (GstGDPPay * this, GstEvent * event)
 {
   GstBuffer *headerbuf;
+
   GstBuffer *payloadbuf;
+
   guint8 *header, *payload;
+
   guint len;
+
   gboolean ret;
 
   ret =
@@ -330,30 +343,44 @@ gst_gdp_pay_reset_streamheader (GstGDPPay * this)
 
   /* We use copies of these to avoid circular refcounts */
   GstBuffer *new_segment_buf, *caps_buf, *tag_buf;
+
   GstStructure *structure;
+
   GstFlowReturn r = GST_FLOW_OK;
+
   gboolean version_one_zero = TRUE;
 
   GValue array = { 0 };
   GValue value = { 0 };
 
+  GST_DEBUG_OBJECT (this, "start");
   /* In version 0.2, we didn't need or send new segment or tags */
   if (this->version == GST_DP_VERSION_0_2)
     version_one_zero = FALSE;
 
   if (version_one_zero) {
-    if (!this->new_segment_buf || !this->caps_buf)
+    if (!this->new_segment_buf || !this->caps_buf) {
+      GST_DEBUG_OBJECT (this, "1.0, missing new_segment or caps, returning");
       return GST_FLOW_OK;
+    }
   } else {
-    if (!this->caps_buf)
+    if (!this->caps_buf) {
+      GST_DEBUG_OBJECT (this, "0.2, missing caps, returning");
       return GST_FLOW_OK;
+    }
   }
 
-  /* put copies of the buffers in a fixed list */
+  /* put copies of the buffers in a fixed list
+   * Stamp the buffers with offset and offset_end as well.
+   * We do this here so the offsets match the order the buffers go out in */
   g_value_init (&array, GST_TYPE_ARRAY);
 
   if (version_one_zero) {
+    gst_gdp_stamp_buffer (this, this->new_segment_buf);
+    GST_DEBUG_OBJECT (this, "1.0, appending copy of new segment buffer %p",
+        this->new_segment_buf);
     new_segment_buf = gst_buffer_copy (this->new_segment_buf);
+    gst_buffer_set_caps (new_segment_buf, NULL);
     g_value_init (&value, GST_TYPE_BUFFER);
     gst_value_set_buffer (&value, new_segment_buf);
     gst_value_array_append_value (&array, &value);
@@ -361,7 +388,11 @@ gst_gdp_pay_reset_streamheader (GstGDPPay * this)
     gst_buffer_unref (new_segment_buf);
 
     if (this->tag_buf) {
+      gst_gdp_stamp_buffer (this, this->tag_buf);
+      GST_DEBUG_OBJECT (this, "1.0, appending copy of tag buffer %p",
+          this->tag_buf);
       tag_buf = gst_buffer_copy (this->tag_buf);
+      gst_buffer_set_caps (tag_buf, NULL);
       g_value_init (&value, GST_TYPE_BUFFER);
       gst_value_set_buffer (&value, tag_buf);
       gst_value_array_append_value (&array, &value);
@@ -370,7 +401,10 @@ gst_gdp_pay_reset_streamheader (GstGDPPay * this)
     }
   }
 
+  gst_gdp_stamp_buffer (this, this->caps_buf);
+  GST_DEBUG_OBJECT (this, "appending copy of caps buffer %p", this->caps_buf);
   caps_buf = gst_buffer_copy (this->caps_buf);
+  gst_buffer_set_caps (caps_buf, NULL);
   g_value_init (&value, GST_TYPE_BUFFER);
   gst_value_set_buffer (&value, caps_buf);
   gst_value_array_append_value (&array, &value);
@@ -382,8 +416,11 @@ gst_gdp_pay_reset_streamheader (GstGDPPay * this)
   structure = gst_caps_get_structure (this->caps, 0);
   if (gst_structure_has_field (structure, "streamheader")) {
     const GValue *sh;
+
     GArray *buffers;
+
     GstBuffer *buffer;
+
     int i;
 
     sh = gst_structure_get_value (structure, "streamheader");
@@ -393,6 +430,7 @@ gst_gdp_pay_reset_streamheader (GstGDPPay * this)
         buffers->len);
     for (i = 0; i < buffers->len; ++i) {
       GValue *bufval;
+
       GstBuffer *outbuffer;
 
       bufval = &g_array_index (buffers, GValue, i);
@@ -410,8 +448,12 @@ gst_gdp_pay_reset_streamheader (GstGDPPay * this)
 
       gst_buffer_unref (outbuffer);
     }
+  } else {
+    GST_DEBUG_OBJECT (this, "no streamheader to serialize");
   }
 
+  GST_DEBUG_OBJECT (this, "%d serialized buffers on streamheaders",
+      gst_value_array_get_size (&array));
   caps = gst_caps_from_string ("application/x-gdp");
   structure = gst_caps_get_structure (caps, 0);
 
@@ -436,8 +478,10 @@ gst_gdp_pay_reset_streamheader (GstGDPPay * this)
   }
 
   /* push out these streamheader buffers, then flush our internal queue */
-  GST_DEBUG_OBJECT (this, "Pushing GDP new_segment buffer %p",
-      this->new_segment_buf);
+  GST_DEBUG_OBJECT (this, "Pushing GDP new_segment buffer %p with offset %"
+      G_GINT64_FORMAT ", offset_end %" G_GINT64_FORMAT, this->new_segment_buf,
+      GST_BUFFER_OFFSET (this->new_segment_buf),
+      GST_BUFFER_OFFSET_END (this->new_segment_buf));
   /* we stored these bufs with refcount 1, so make sure we keep a ref */
   r = gst_pad_push (this->srcpad, gst_buffer_ref (this->new_segment_buf));
   if (r != GST_FLOW_OK) {
@@ -482,6 +526,7 @@ gst_gdp_pay_reset_streamheader (GstGDPPay * this)
 
 done:
   gst_caps_unref (caps);
+  GST_DEBUG_OBJECT (this, "stop");
   return r;
 
   /* ERRORS */
@@ -506,7 +551,8 @@ gst_gdp_queue_buffer (GstGDPPay * this, GstBuffer * buffer)
 
   /* store it on an internal queue. buffer remains reffed. */
   this->queue = g_list_append (this->queue, buffer);
-  GST_DEBUG_OBJECT (this, "queued buffer %p, now %d buffers queued",
+  GST_DEBUG_OBJECT (this, "streamheader not sent yet, "
+      "queued buffer %p, now %d buffers queued",
       buffer, g_list_length (this->queue));
 
   return GST_FLOW_OK;
@@ -516,8 +562,11 @@ static GstFlowReturn
 gst_gdp_pay_chain (GstPad * pad, GstBuffer * buffer)
 {
   GstGDPPay *this;
+
   GstCaps *caps;
+
   GstBuffer *outbuffer;
+
   GstFlowReturn ret;
 
   this = GST_GDP_PAY (gst_pad_get_parent (pad));
@@ -538,9 +587,9 @@ gst_gdp_pay_chain (GstPad * pad, GstBuffer * buffer)
       GST_ELEMENT_WARNING (this, STREAM, ENCODE, (NULL),
           ("Could not create GDP buffer from new segment event"));
     } else {
-      gst_gdp_stamp_buffer (this, outbuffer);
       GST_BUFFER_TIMESTAMP (outbuffer) = GST_BUFFER_TIMESTAMP (buffer);
       GST_BUFFER_DURATION (outbuffer) = 0;
+      GST_BUFFER_FLAG_SET (outbuffer, GST_BUFFER_FLAG_IN_CAPS);
       GST_DEBUG_OBJECT (this, "Storing buffer %p as new_segment_buf",
           outbuffer);
       this->new_segment_buf = outbuffer;
@@ -560,10 +609,12 @@ gst_gdp_pay_chain (GstPad * pad, GstBuffer * buffer)
     if (!outbuffer)
       goto no_caps_buffer;
 
-    gst_gdp_stamp_buffer (this, outbuffer);
     GST_BUFFER_TIMESTAMP (outbuffer) = GST_BUFFER_TIMESTAMP (buffer);
     GST_BUFFER_DURATION (outbuffer) = 0;
     GST_BUFFER_FLAG_SET (outbuffer, GST_BUFFER_FLAG_IN_CAPS);
+
+    if (this->caps_buf)
+      gst_buffer_unref (this->caps_buf);
     this->caps_buf = outbuffer;
     gst_gdp_pay_reset_streamheader (this);
   }
@@ -577,6 +628,15 @@ gst_gdp_pay_chain (GstPad * pad, GstBuffer * buffer)
   if (!outbuffer)
     goto no_buffer;
 
+  /* If the incoming buffer is IN_CAPS, that means we have it on the caps
+   * as streamheader, and we have serialized a GDP version of it and put it
+   * on our caps */
+  if (GST_BUFFER_FLAG_IS_SET (buffer, GST_BUFFER_FLAG_IN_CAPS)) {
+    GST_DEBUG_OBJECT (this, "Setting IN_CAPS flag on outgoing buffer %p",
+        outbuffer);
+    GST_BUFFER_FLAG_SET (outbuffer, GST_BUFFER_FLAG_IN_CAPS);
+  }
+
   gst_gdp_stamp_buffer (this, outbuffer);
   GST_BUFFER_TIMESTAMP (outbuffer) = GST_BUFFER_TIMESTAMP (buffer);
   GST_BUFFER_DURATION (outbuffer) = GST_BUFFER_DURATION (buffer);
@@ -621,8 +681,11 @@ static gboolean
 gst_gdp_pay_sink_event (GstPad * pad, GstEvent * event)
 {
   GstBuffer *outbuffer;
+
   GstGDPPay *this = GST_GDP_PAY (gst_pad_get_parent (pad));
+
   GstFlowReturn flowret;
+
   gboolean ret = TRUE;
 
   GST_DEBUG_OBJECT (this, "received event %p of type %s (%d)",
@@ -633,30 +696,32 @@ gst_gdp_pay_sink_event (GstPad * pad, GstEvent * event)
   if (!outbuffer)
     goto no_outbuffer;
 
-  gst_gdp_stamp_buffer (this, outbuffer);
   GST_BUFFER_TIMESTAMP (outbuffer) = GST_EVENT_TIMESTAMP (event);
   GST_BUFFER_DURATION (outbuffer) = 0;
 
-  /* if we got a new segment, we should put it on our streamheader,
+  /* if we got a new segment or tag event, we should put it on our streamheader,
    * and not send it on */
   switch (GST_EVENT_TYPE (event)) {
     case GST_EVENT_NEWSEGMENT:
-      GST_DEBUG_OBJECT (this, "Storing buffer %p as new_segment_buf",
+      GST_DEBUG_OBJECT (this, "Storing in caps buffer %p as new_segment_buf",
           outbuffer);
 
       if (this->new_segment_buf)
         gst_buffer_unref (this->new_segment_buf);
       this->new_segment_buf = outbuffer;
 
+      GST_BUFFER_FLAG_SET (outbuffer, GST_BUFFER_FLAG_IN_CAPS);
       gst_gdp_pay_reset_streamheader (this);
       break;
     case GST_EVENT_TAG:
-      GST_DEBUG_OBJECT (this, "Storing buffer %p as tag_buf", outbuffer);
+      GST_DEBUG_OBJECT (this, "Storing in caps buffer %p as tag_buf",
+          outbuffer);
 
       if (this->tag_buf)
         gst_buffer_unref (this->tag_buf);
       this->tag_buf = outbuffer;
 
+      GST_BUFFER_FLAG_SET (outbuffer, GST_BUFFER_FLAG_IN_CAPS);
       gst_gdp_pay_reset_streamheader (this);
       break;
     default:
@@ -702,6 +767,7 @@ static gboolean
 gst_gdp_pay_src_event (GstPad * pad, GstEvent * event)
 {
   GstGDPPay *this;
+
   gboolean res = TRUE;
 
   this = GST_GDP_PAY (gst_pad_get_parent (pad));
@@ -782,6 +848,7 @@ static GstStateChangeReturn
 gst_gdp_pay_change_state (GstElement * element, GstStateChange transition)
 {
   GstStateChangeReturn ret;
+
   GstGDPPay *this = GST_GDP_PAY (element);
 
   switch (transition) {
