@@ -1,7 +1,7 @@
 /* GStreamer
  * Copyright (C) 1999,2000 Erik Walthinsen <omega@cse.ogi.edu>
  *                    2003 Colin Walters <cwalters@gnome.org>
- *                    2000,2005,2007 Wim Taymans <wim@fluendo.com>
+ *                    2000,2005,2007 Wim Taymans <wim.taymans@gmail.com>
  *                    2007 Thiago Sousa Santos <thiagoss@lcc.ufcg.edu.br>
  *
  * gstqueue2.c:
@@ -60,7 +60,8 @@
 static const GstElementDetails gst_queue_details = GST_ELEMENT_DETAILS ("Queue",
     "Generic",
     "Simple data queue",
-    "Erik Walthinsen <omega@cse.ogi.edu>, " "Wim Taymans <wim@fluendo.com>");
+    "Erik Walthinsen <omega@cse.ogi.edu>, "
+    "Wim Taymans <wim.taymans@gmail.com>");
 
 static GstStaticPadTemplate sinktemplate = GST_STATIC_PAD_TEMPLATE ("sink",
     GST_PAD_SINK,
@@ -176,13 +177,17 @@ struct _GstQueue
   guint buffering_iteration;
 
   /* for measuring input/output rates */
+  GTimer *in_timer;
+  gboolean in_timer_started;
+  gdouble last_in_elapsed;
   guint64 bytes_in;
-  guint64 bytes_out;
-  GTimer *timer;
   gdouble byte_in_rate;
+
+  GTimer *out_timer;
+  gboolean out_timer_started;
+  gdouble last_out_elapsed;
+  guint64 bytes_out;
   gdouble byte_out_rate;
-  gdouble last_elapsed;
-  gboolean timer_started;
 
   GMutex *qlock;                /* lock for queue (vs object lock) */
   gboolean waiting_add;
@@ -359,50 +364,53 @@ gst_queue_class_init (GstQueueClass * klass)
   g_object_class_install_property (gobject_class, PROP_CUR_LEVEL_BYTES,
       g_param_spec_uint ("current-level-bytes", "Current level (kB)",
           "Current amount of data in the queue (bytes)",
-          0, G_MAXUINT, 0, G_PARAM_READABLE));
+          0, G_MAXUINT, 0, G_PARAM_READABLE | G_PARAM_STATIC_STRINGS));
   g_object_class_install_property (gobject_class, PROP_CUR_LEVEL_BUFFERS,
       g_param_spec_uint ("current-level-buffers", "Current level (buffers)",
           "Current number of buffers in the queue",
-          0, G_MAXUINT, 0, G_PARAM_READABLE));
+          0, G_MAXUINT, 0, G_PARAM_READABLE | G_PARAM_STATIC_STRINGS));
   g_object_class_install_property (gobject_class, PROP_CUR_LEVEL_TIME,
       g_param_spec_uint64 ("current-level-time", "Current level (ns)",
           "Current amount of data in the queue (in ns)",
-          0, G_MAXUINT64, 0, G_PARAM_READABLE));
+          0, G_MAXUINT64, 0, G_PARAM_READABLE | G_PARAM_STATIC_STRINGS));
 
   g_object_class_install_property (gobject_class, PROP_MAX_SIZE_BYTES,
       g_param_spec_uint ("max-size-bytes", "Max. size (kB)",
           "Max. amount of data in the queue (bytes, 0=disable)",
-          0, G_MAXUINT, DEFAULT_MAX_SIZE_BYTES, G_PARAM_READWRITE));
+          0, G_MAXUINT, DEFAULT_MAX_SIZE_BYTES,
+          G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
   g_object_class_install_property (gobject_class, PROP_MAX_SIZE_BUFFERS,
       g_param_spec_uint ("max-size-buffers", "Max. size (buffers)",
-          "Max. number of buffers in the queue (0=disable)",
-          0, G_MAXUINT, DEFAULT_MAX_SIZE_BUFFERS, G_PARAM_READWRITE));
+          "Max. number of buffers in the queue (0=disable)", 0, G_MAXUINT,
+          DEFAULT_MAX_SIZE_BUFFERS,
+          G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
   g_object_class_install_property (gobject_class, PROP_MAX_SIZE_TIME,
       g_param_spec_uint64 ("max-size-time", "Max. size (ns)",
-          "Max. amount of data in the queue (in ns, 0=disable)",
-          0, G_MAXUINT64, DEFAULT_MAX_SIZE_TIME, G_PARAM_READWRITE));
+          "Max. amount of data in the queue (in ns, 0=disable)", 0, G_MAXUINT64,
+          DEFAULT_MAX_SIZE_TIME, G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
 
   g_object_class_install_property (gobject_class, PROP_USE_BUFFERING,
       g_param_spec_boolean ("use-buffering", "Use buffering",
           "Emit GST_MESSAGE_BUFFERING based on low-/high-percent thresholds",
-          DEFAULT_USE_BUFFERING, G_PARAM_READWRITE));
+          DEFAULT_USE_BUFFERING, G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
   g_object_class_install_property (gobject_class, PROP_USE_RATE_ESTIMATE,
       g_param_spec_boolean ("use-rate-estimate", "Use Rate Estimate",
           "Estimate the bitrate of the stream to calculate time level",
-          DEFAULT_USE_RATE_ESTIMATE, G_PARAM_READWRITE));
+          DEFAULT_USE_RATE_ESTIMATE,
+          G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
   g_object_class_install_property (gobject_class, PROP_LOW_PERCENT,
       g_param_spec_int ("low-percent", "Low percent",
-          "Low threshold for buffering to start",
-          0, 100, DEFAULT_LOW_PERCENT, G_PARAM_READWRITE));
+          "Low threshold for buffering to start", 0, 100, DEFAULT_LOW_PERCENT,
+          G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
   g_object_class_install_property (gobject_class, PROP_HIGH_PERCENT,
       g_param_spec_int ("high-percent", "High percent",
-          "High threshold for buffering to finish",
-          0, 100, DEFAULT_HIGH_PERCENT, G_PARAM_READWRITE));
+          "High threshold for buffering to finish", 0, 100,
+          DEFAULT_HIGH_PERCENT, G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
 
   g_object_class_install_property (gobject_class, PROP_TEMP_LOCATION,
       g_param_spec_string ("temp-location", "Temporary File Location",
           "Location of a temporary file to store data in",
-          NULL, G_PARAM_READWRITE));
+          NULL, G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
 
   gst_element_class_add_pad_template (gstelement_class,
       gst_static_pad_template_get (&srctemplate));
@@ -468,7 +476,8 @@ gst_queue_init (GstQueue * queue, GstQueueClass * g_class)
 
   queue->srcresult = GST_FLOW_WRONG_STATE;
   queue->is_eos = FALSE;
-  queue->timer = g_timer_new ();
+  queue->in_timer = g_timer_new ();
+  queue->out_timer = g_timer_new ();
 
   queue->qlock = g_mutex_new ();
   queue->waiting_add = FALSE;
@@ -503,7 +512,8 @@ gst_queue_finalize (GObject * object)
   g_mutex_free (queue->qlock);
   g_cond_free (queue->item_add);
   g_cond_free (queue->item_del);
-  g_timer_destroy (queue->timer);
+  g_timer_destroy (queue->in_timer);
+  g_timer_destroy (queue->out_timer);
 
   /* temp_file path cleanup  */
   if (queue->temp_location != NULL)
@@ -679,15 +689,27 @@ update_buffering (GstQueue * queue)
     }
   }
   if (post) {
+    GstMessage *message;
+    GstBufferingMode mode;
+
     /* scale to high percent so that it becomes the 100% mark */
     percent = percent * 100 / queue->high_percent;
     /* clip */
     if (percent > 100)
       percent = 100;
 
+    if (QUEUE_IS_USING_TEMP_FILE (queue))
+      mode = GST_BUFFERING_DOWNLOAD;
+    else
+      mode = GST_BUFFERING_STREAM;
+
     GST_DEBUG_OBJECT (queue, "buffering %d percent", percent);
-    gst_element_post_message (GST_ELEMENT_CAST (queue),
-        gst_message_new_buffering (GST_OBJECT_CAST (queue), percent));
+    message = gst_message_new_buffering (GST_OBJECT_CAST (queue), percent);
+    gst_message_set_buffering_stats (message, mode,
+        queue->byte_in_rate, queue->byte_out_rate, -1);
+
+    gst_element_post_message (GST_ELEMENT_CAST (queue), message);
+
   } else {
     GST_DEBUG_OBJECT (queue, "filled %d percent", percent);
   }
@@ -702,8 +724,10 @@ reset_rate_timer (GstQueue * queue)
   queue->bytes_out = 0;
   queue->byte_in_rate = 0.0;
   queue->byte_out_rate = 0.0;
-  queue->last_elapsed = 0.0;
-  queue->timer_started = FALSE;
+  queue->last_in_elapsed = 0.0;
+  queue->last_out_elapsed = 0.0;
+  queue->in_timer_started = FALSE;
+  queue->out_timer_started = FALSE;
 }
 
 /* the interval in seconds to recalculate the rate */
@@ -716,55 +740,84 @@ reset_rate_timer (GstQueue * queue)
 #define AVG_OUT(avg,val) ((avg) * 3.0 + (val)) / 4.0
 
 static void
-update_rates (GstQueue * queue)
+update_in_rates (GstQueue * queue)
 {
   gdouble elapsed, period;
   gdouble byte_in_rate;
-  gdouble byte_out_rate;
 
-  if (!queue->timer_started) {
-    queue->timer_started = TRUE;
-    g_timer_start (queue->timer);
+  if (!queue->in_timer_started) {
+    queue->in_timer_started = TRUE;
+    g_timer_start (queue->in_timer);
     return;
   }
 
-  elapsed = g_timer_elapsed (queue->timer, NULL);
+  elapsed = g_timer_elapsed (queue->in_timer, NULL);
 
   /* recalc after each interval. */
-  if (queue->last_elapsed + RATE_INTERVAL < elapsed) {
-    period = elapsed - queue->last_elapsed;
+  if (queue->last_in_elapsed + RATE_INTERVAL < elapsed) {
+    period = elapsed - queue->last_in_elapsed;
 
     GST_DEBUG_OBJECT (queue,
-        "rates: period %f, in %" G_GUINT64_FORMAT ", out %" G_GUINT64_FORMAT,
-        period, queue->bytes_in, queue->bytes_out);
+        "rates: period %f, in %" G_GUINT64_FORMAT, period, queue->bytes_in);
 
     byte_in_rate = queue->bytes_in / period;
-    byte_out_rate = queue->bytes_out / period;
 
     if (queue->byte_in_rate == 0.0)
       queue->byte_in_rate = byte_in_rate;
     else
       queue->byte_in_rate = AVG_IN (queue->byte_in_rate, byte_in_rate);
 
+    /* reset the values to calculate rate over the next interval */
+    queue->last_in_elapsed = elapsed;
+    queue->bytes_in = 0;
+  }
+
+  if (queue->byte_in_rate > 0.0) {
+    queue->cur_level.rate_time =
+        queue->cur_level.bytes / queue->byte_in_rate * GST_SECOND;
+  }
+  GST_DEBUG_OBJECT (queue, "rates: in %f, time %" GST_TIME_FORMAT,
+      queue->byte_in_rate, GST_TIME_ARGS (queue->cur_level.rate_time));
+}
+
+static void
+update_out_rates (GstQueue * queue)
+{
+  gdouble elapsed, period;
+  gdouble byte_out_rate;
+
+  if (!queue->out_timer_started) {
+    queue->out_timer_started = TRUE;
+    g_timer_start (queue->out_timer);
+    return;
+  }
+
+  elapsed = g_timer_elapsed (queue->out_timer, NULL);
+
+  /* recalc after each interval. */
+  if (queue->last_out_elapsed + RATE_INTERVAL < elapsed) {
+    period = elapsed - queue->last_out_elapsed;
+
+    GST_DEBUG_OBJECT (queue,
+        "rates: period %f, out %" G_GUINT64_FORMAT, period, queue->bytes_out);
+
+    byte_out_rate = queue->bytes_out / period;
+
     if (queue->byte_out_rate == 0.0)
       queue->byte_out_rate = byte_out_rate;
     else
       queue->byte_out_rate = AVG_OUT (queue->byte_out_rate, byte_out_rate);
 
     /* reset the values to calculate rate over the next interval */
-    queue->last_elapsed = elapsed;
-    queue->bytes_in = 0;
+    queue->last_out_elapsed = elapsed;
     queue->bytes_out = 0;
   }
-
   if (queue->byte_in_rate > 0.0) {
     queue->cur_level.rate_time =
         queue->cur_level.bytes / queue->byte_in_rate * GST_SECOND;
   }
-
-  GST_DEBUG_OBJECT (queue, "rates: in %f, out %f, time %" GST_TIME_FORMAT,
-      queue->byte_in_rate, queue->byte_out_rate,
-      GST_TIME_ARGS (queue->cur_level.rate_time));
+  GST_DEBUG_OBJECT (queue, "rates: out %f, time %" GST_TIME_FORMAT,
+      queue->byte_out_rate, GST_TIME_ARGS (queue->cur_level.rate_time));
 }
 
 static void
@@ -918,6 +971,8 @@ gst_queue_open_temp_location_file (GstQueue * queue)
   if (queue->temp_location == NULL)
     goto no_filename;
 
+  GST_DEBUG_OBJECT (queue, "opening temp file %s", queue->temp_location);
+
   /* open the file for update/writing */
   queue->temp_file = g_fopen (queue->temp_location, "wb+");
   /* error creating file */
@@ -952,6 +1007,8 @@ gst_queue_close_temp_location_file (GstQueue * queue)
   if (queue->temp_file == NULL)
     return;
 
+  GST_DEBUG_OBJECT (queue, "closing temp file");
+
   /* we don't remove the file so that the application can use it as a cache
    * later on */
   fflush (queue->temp_file);
@@ -1005,7 +1062,7 @@ gst_queue_locked_enqueue (GstQueue * queue, gpointer item)
     /* apply new buffer to segment stats */
     apply_buffer (queue, buffer, &queue->sink_segment);
     /* update the byterate stats */
-    update_rates (queue);
+    update_in_rates (queue);
 
     if (QUEUE_IS_USING_TEMP_FILE (queue)) {
       gst_queue_write_buffer_to_file (queue, buffer);
@@ -1101,7 +1158,7 @@ gst_queue_locked_dequeue (GstQueue * queue)
     queue->bytes_out += size;
     apply_buffer (queue, buffer, &queue->src_segment);
     /* update the byterate stats */
-    update_rates (queue);
+    update_out_rates (queue);
     /* update the buffering */
     update_buffering (queue);
 
@@ -1251,6 +1308,10 @@ gst_queue_is_filled (GstQueue * queue)
   if (QUEUE_IS_USING_TEMP_FILE (queue))
     return FALSE;
 
+  /* we are never filled when we have no buffers at all */
+  if (queue->cur_level.buffers == 0)
+    return FALSE;
+
 #define CHECK_FILLED(format) ((queue->max_level.format) > 0 && \
 		(queue->cur_level.format) >= (queue->max_level.format))
 
@@ -1290,11 +1351,25 @@ gst_queue_chain (GstPad * pad, GstBuffer * buffer)
 
   /* We make space available if we're "full" according to whatever
    * the user defined as "full". */
-  while (gst_queue_is_filled (queue)) {
+  if (gst_queue_is_filled (queue)) {
+    gboolean started;
+
+    /* pause the timer while we wait. The fact that we are waiting does not mean
+     * the byterate on the input pad is lower */
+    if ((started = queue->in_timer_started))
+      g_timer_stop (queue->in_timer);
+
     GST_CAT_DEBUG_OBJECT (queue_dataflow, queue,
         "queue is full, waiting for free space");
-    /* Wait for space to be available, we could be unlocked because of a flush. */
-    GST_QUEUE_WAIT_DEL_CHECK (queue, out_flushing);
+    do {
+      /* Wait for space to be available, we could be unlocked because of a flush. */
+      GST_QUEUE_WAIT_DEL_CHECK (queue, out_flushing);
+    }
+    while (gst_queue_is_filled (queue));
+
+    /* and continue if we were running before */
+    if (started)
+      g_timer_continue (queue->in_timer);
   }
 
   /* put buffer in queue now */
@@ -1446,9 +1521,25 @@ gst_queue_loop (GstPad * pad)
   /* have to lock for thread-safety */
   GST_QUEUE_MUTEX_LOCK_CHECK (queue, out_flushing);
 
-  while (gst_queue_is_empty (queue)) {
-    /* we recheck, we could be unlocked because of a flush. */
-    GST_QUEUE_WAIT_ADD_CHECK (queue, out_flushing);
+  if (gst_queue_is_empty (queue)) {
+    gboolean started;
+
+    /* pause the timer while we wait. The fact that we are waiting does not mean
+     * the byterate on the output pad is lower */
+    if ((started = queue->out_timer_started))
+      g_timer_stop (queue->out_timer);
+
+    GST_CAT_DEBUG_OBJECT (queue_dataflow, queue,
+        "queue is empty, waiting for new data");
+    do {
+      /* Wait for data to be available, we could be unlocked because of a flush. */
+      GST_QUEUE_WAIT_ADD_CHECK (queue, out_flushing);
+    }
+    while (gst_queue_is_empty (queue));
+
+    /* and continue if we were running before */
+    if (started)
+      g_timer_continue (queue->out_timer);
   }
   ret = gst_queue_push_one (queue);
   queue->srcresult = ret;
@@ -1481,8 +1572,14 @@ gst_queue_handle_src_event (GstPad * pad, GstEvent * event)
       event, GST_EVENT_TYPE_NAME (event));
 #endif
 
-  /* just forward upstream */
-  res = gst_pad_push_event (queue->sinkpad, event);
+  if (!QUEUE_IS_USING_TEMP_FILE (queue)) {
+    /* just forward upstream */
+    res = gst_pad_push_event (queue->sinkpad, event);
+  } else {
+    /* when using a temp file, we unblock the pending read */
+    res = TRUE;
+    gst_event_unref (event);
+  }
 
   return res;
 }
@@ -1538,15 +1635,7 @@ gst_queue_handle_src_query (GstPad * pad, GstQuery * query)
     }
     case GST_QUERY_DURATION:
     {
-      GST_DEBUG_OBJECT (queue, "waiting for preroll in duration query");
-
-      GST_QUEUE_MUTEX_LOCK (queue);
-      /* we have to wait until the upstream element is at least paused, which
-       * happened when we received a first item. */
-      while (gst_queue_is_empty (queue)) {
-        GST_QUEUE_WAIT_ADD_CHECK (queue, flushing);
-      }
-      GST_QUEUE_MUTEX_UNLOCK (queue);
+      GST_DEBUG_OBJECT (queue, "doing peer query");
 
       if (!gst_queue_peer_query (queue, queue->sinkpad, query))
         goto peer_failed;
@@ -1554,6 +1643,59 @@ gst_queue_handle_src_query (GstPad * pad, GstQuery * query)
       GST_DEBUG_OBJECT (queue, "peer query success");
       break;
     }
+    case GST_QUERY_BUFFERING:
+    {
+      GstFormat format;
+
+      GST_DEBUG_OBJECT (queue, "query buffering");
+
+      if (!QUEUE_IS_USING_TEMP_FILE (queue)) {
+        /* no temp file, just forward to the peer */
+        if (!gst_queue_peer_query (queue, queue->sinkpad, query))
+          goto peer_failed;
+        GST_DEBUG_OBJECT (queue, "buffering forwarded to peer");
+      } else {
+        gint64 start, stop;
+
+        gst_query_parse_buffering_range (query, &format, NULL, NULL, NULL);
+
+        switch (format) {
+          case GST_FORMAT_PERCENT:
+          {
+            gint64 duration;
+            GstFormat peer_fmt;
+
+            peer_fmt = GST_FORMAT_BYTES;
+
+            if (!gst_pad_query_peer_duration (queue->sinkpad, &peer_fmt,
+                    &duration))
+              goto peer_failed;
+
+            GST_DEBUG_OBJECT (queue, "duration %" G_GINT64_FORMAT ", writing %"
+                G_GINT64_FORMAT, duration, queue->writing_pos);
+
+            start = 0;
+            /* get our available data relative to the duration */
+            if (duration != -1)
+              stop = GST_FORMAT_PERCENT_MAX * queue->writing_pos / duration;
+            else
+              stop = -1;
+            break;
+          }
+          case GST_FORMAT_BYTES:
+            start = 0;
+            stop = queue->writing_pos;
+            break;
+          default:
+            start = -1;
+            stop = -1;
+            break;
+        }
+        gst_query_set_buffering_percent (query, queue->is_buffering, 100);
+        gst_query_set_buffering_range (query, format, start, stop, -1);
+      }
+      break;
+    }
     default:
       /* peer handled other queries */
       if (!gst_queue_peer_query (queue, queue->sinkpad, query))
@@ -1569,12 +1711,6 @@ peer_failed:
     GST_DEBUG_OBJECT (queue, "failed peer query");
     return FALSE;
   }
-flushing:
-  {
-    GST_DEBUG_OBJECT (queue, "flushing while waiting for query");
-    GST_QUEUE_MUTEX_UNLOCK (queue);
-    return FALSE;
-  }
 }
 
 static GstFlowReturn
@@ -1614,8 +1750,10 @@ gst_queue_src_checkgetrange_function (GstPad * pad)
   gboolean ret;
 
   queue = GST_QUEUE (gst_pad_get_parent (pad));
+
   /* we can operate in pull mode when we are using a tempfile */
   ret = QUEUE_IS_USING_TEMP_FILE (queue);
+
   gst_object_unref (GST_OBJECT (queue));
 
   return ret;
@@ -1857,7 +1995,7 @@ gst_queue_set_property (GObject * object,
       queue->high_percent = g_value_get_int (value);
       break;
     case PROP_TEMP_LOCATION:
-      gst_queue_set_temp_location (queue, g_value_dup_string (value));
+      gst_queue_set_temp_location (queue, g_value_get_string (value));
       break;
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
