@@ -117,11 +117,16 @@
 #include <gst/gst-i18n-plugin.h>
 
 #include <sys/ioctl.h>
+
+#ifdef HAVE_UNISTD_H
 #include <unistd.h>
+#endif
+
 #include <fcntl.h>
 #include <sys/types.h>
 #include <sys/socket.h>
 #include <sys/stat.h>
+#include <netinet/in.h>
 
 #ifdef HAVE_FIONREAD_IN_SYS_FILIO
 #include <sys/filio.h>
@@ -132,25 +137,6 @@
 
 #define NOT_IMPLEMENTED 0
 
-/* the select call is also performed on the control sockets, that way
- * we can send special commands to unblock or restart the select call */
-#define CONTROL_RESTART         'R'     /* restart the select call */
-#define CONTROL_STOP            'S'     /* stop the select call */
-#define CONTROL_SOCKETS(sink)   sink->control_sock
-#define WRITE_SOCKET(sink)      sink->control_sock[1]
-#define READ_SOCKET(sink)       sink->control_sock[0]
-
-#define SEND_COMMAND(sink, command)             \
-G_STMT_START {                                  \
-  unsigned char c; c = command;                 \
-  write (WRITE_SOCKET(sink).fd, &c, 1);         \
-} G_STMT_END
-
-#define READ_COMMAND(sink, command, res)        \
-G_STMT_START {                                  \
-  res = read(READ_SOCKET(sink).fd, &command, 1);\
-} G_STMT_END
-
 /* elementfactory information */
 static const GstElementDetails gst_multi_fd_sink_details =
 GST_ELEMENT_DETAILS ("Multi filedescriptor sink",
@@ -186,24 +172,27 @@ enum
   LAST_SIGNAL
 };
 
+
 /* this is really arbitrarily chosen */
 #define DEFAULT_PROTOCOL                GST_TCP_PROTOCOL_NONE
-#define DEFAULT_MODE                    GST_FDSET_MODE_POLL
+#define DEFAULT_MODE                    1
 #define DEFAULT_BUFFERS_MAX             -1
 #define DEFAULT_BUFFERS_SOFT_MAX        -1
 #define DEFAULT_TIME_MIN                -1
 #define DEFAULT_BYTES_MIN               -1
 #define DEFAULT_BUFFERS_MIN             -1
-#define DEFAULT_UNIT_TYPE               GST_UNIT_TYPE_BUFFERS
+#define DEFAULT_UNIT_TYPE               GST_TCP_UNIT_TYPE_BUFFERS
 #define DEFAULT_UNITS_MAX               -1
 #define DEFAULT_UNITS_SOFT_MAX          -1
 #define DEFAULT_RECOVER_POLICY          GST_RECOVER_POLICY_NONE
 #define DEFAULT_TIMEOUT                 0
 #define DEFAULT_SYNC_METHOD             GST_SYNC_METHOD_LATEST
 
-#define DEFAULT_BURST_UNIT              GST_UNIT_TYPE_UNDEFINED
+#define DEFAULT_BURST_UNIT              GST_TCP_UNIT_TYPE_UNDEFINED
 #define DEFAULT_BURST_VALUE             0
 
+#define DEFAULT_QOS_DSCP                -1
+
 enum
 {
   PROP_0,
@@ -232,8 +221,32 @@ enum
 
   PROP_BURST_UNIT,
   PROP_BURST_VALUE,
+
+  PROP_QOS_DSCP,
+
+  PROP_LAST
 };
 
+/* For backward compat, we can't really select the poll mode anymore with
+ * GstPoll. */
+#define GST_TYPE_FDSET_MODE (gst_fdset_mode_get_type())
+static GType
+gst_fdset_mode_get_type (void)
+{
+  static GType fdset_mode_type = 0;
+  static const GEnumValue fdset_mode[] = {
+    {0, "Select", "select"},
+    {1, "Poll", "poll"},
+    {2, "EPoll", "epoll"},
+    {0, NULL, NULL},
+  };
+
+  if (!fdset_mode_type) {
+    fdset_mode_type = g_enum_register_static ("GstFDSetMode", fdset_mode);
+  }
+  return fdset_mode_type;
+}
+
 #define GST_TYPE_RECOVER_POLICY (gst_recover_policy_get_type())
 static GType
 gst_recover_policy_get_type (void)
@@ -293,10 +306,10 @@ gst_unit_type_get_type (void)
 {
   static GType unit_type_type = 0;
   static const GEnumValue unit_type[] = {
-    {GST_UNIT_TYPE_UNDEFINED, "Undefined", "undefined"},
-    {GST_UNIT_TYPE_BUFFERS, "Buffers", "buffers"},
-    {GST_UNIT_TYPE_BYTES, "Bytes", "bytes"},
-    {GST_UNIT_TYPE_TIME, "Time", "time"},
+    {GST_TCP_UNIT_TYPE_UNDEFINED, "Undefined", "undefined"},
+    {GST_TCP_UNIT_TYPE_BUFFERS, "Buffers", "buffers"},
+    {GST_TCP_UNIT_TYPE_BYTES, "Bytes", "bytes"},
+    {GST_TCP_UNIT_TYPE_TIME, "Time", "time"},
     {0, NULL, NULL},
   };
 
@@ -377,92 +390,117 @@ gst_multi_fd_sink_class_init (GstMultiFdSinkClass * klass)
 
   g_object_class_install_property (gobject_class, PROP_PROTOCOL,
       g_param_spec_enum ("protocol", "Protocol", "The protocol to wrap data in",
-          GST_TYPE_TCP_PROTOCOL, DEFAULT_PROTOCOL, G_PARAM_READWRITE));
+          GST_TYPE_TCP_PROTOCOL, DEFAULT_PROTOCOL,
+          G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
+
+  /**
+   * GstMultiFdSink::mode
+   *
+   * The mode for selecting activity on the fds. 
+   *
+   * This property is deprecated since 0.10.18, if will now automatically
+   * select and use the most optimal method.
+   */
   g_object_class_install_property (gobject_class, PROP_MODE,
       g_param_spec_enum ("mode", "Mode",
-          "The mode for selecting activity on the fds", GST_TYPE_FDSET_MODE,
-          DEFAULT_MODE, G_PARAM_READWRITE));
+          "The mode for selecting activity on the fds (deprecated)",
+          GST_TYPE_FDSET_MODE, DEFAULT_MODE,
+          G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
 
   g_object_class_install_property (gobject_class, PROP_BUFFERS_MAX,
       g_param_spec_int ("buffers-max", "Buffers max",
           "max number of buffers to queue for a client (-1 = no limit)", -1,
-          G_MAXINT, DEFAULT_BUFFERS_MAX, G_PARAM_READWRITE));
-  g_object_class_install_property (gobject_class,
-      PROP_BUFFERS_SOFT_MAX, g_param_spec_int ("buffers-soft-max",
-          "Buffers soft max",
+          G_MAXINT, DEFAULT_BUFFERS_MAX,
+          G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
+  g_object_class_install_property (gobject_class, PROP_BUFFERS_SOFT_MAX,
+      g_param_spec_int ("buffers-soft-max", "Buffers soft max",
           "Recover client when going over this limit (-1 = no limit)", -1,
-          G_MAXINT, DEFAULT_BUFFERS_SOFT_MAX, G_PARAM_READWRITE));
+          G_MAXINT, DEFAULT_BUFFERS_SOFT_MAX,
+          G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
 
   g_object_class_install_property (gobject_class, PROP_BYTES_MIN,
       g_param_spec_int ("bytes-min", "Bytes min",
           "min number of bytes to queue (-1 = as little as possible)", -1,
-          G_MAXINT, DEFAULT_BYTES_MIN, G_PARAM_READWRITE));
+          G_MAXINT, DEFAULT_BYTES_MIN,
+          G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
   g_object_class_install_property (gobject_class, PROP_TIME_MIN,
       g_param_spec_int64 ("time-min", "Time min",
           "min number of time to queue (-1 = as little as possible)", -1,
-          G_MAXINT64, DEFAULT_TIME_MIN, G_PARAM_READWRITE));
+          G_MAXINT64, DEFAULT_TIME_MIN,
+          G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
   g_object_class_install_property (gobject_class, PROP_BUFFERS_MIN,
       g_param_spec_int ("buffers-min", "Buffers min",
           "min number of buffers to queue (-1 = as few as possible)", -1,
-          G_MAXINT, DEFAULT_BUFFERS_MIN, G_PARAM_READWRITE));
+          G_MAXINT, DEFAULT_BUFFERS_MIN,
+          G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
 
   g_object_class_install_property (gobject_class, PROP_UNIT_TYPE,
       g_param_spec_enum ("unit-type", "Units type",
           "The unit to measure the max/soft-max/queued properties",
-          GST_TYPE_UNIT_TYPE, DEFAULT_UNIT_TYPE, G_PARAM_READWRITE));
+          GST_TYPE_UNIT_TYPE, DEFAULT_UNIT_TYPE,
+          G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
   g_object_class_install_property (gobject_class, PROP_UNITS_MAX,
       g_param_spec_int64 ("units-max", "Units max",
           "max number of units to queue (-1 = no limit)", -1, G_MAXINT64,
-          DEFAULT_UNITS_MAX, G_PARAM_READWRITE));
+          DEFAULT_UNITS_MAX, G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
   g_object_class_install_property (gobject_class, PROP_UNITS_SOFT_MAX,
       g_param_spec_int64 ("units-soft-max", "Units soft max",
           "Recover client when going over this limit (-1 = no limit)", -1,
-          G_MAXINT64, DEFAULT_UNITS_SOFT_MAX, G_PARAM_READWRITE));
+          G_MAXINT64, DEFAULT_UNITS_SOFT_MAX,
+          G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
 
   g_object_class_install_property (gobject_class, PROP_BUFFERS_QUEUED,
       g_param_spec_uint ("buffers-queued", "Buffers queued",
           "Number of buffers currently queued", 0, G_MAXUINT, 0,
-          G_PARAM_READABLE));
+          G_PARAM_READABLE | G_PARAM_STATIC_STRINGS));
 #if NOT_IMPLEMENTED
   g_object_class_install_property (gobject_class, PROP_BYTES_QUEUED,
       g_param_spec_uint ("bytes-queued", "Bytes queued",
           "Number of bytes currently queued", 0, G_MAXUINT, 0,
-          G_PARAM_READABLE));
+          G_PARAM_READABLE | G_PARAM_STATIC_STRINGS));
   g_object_class_install_property (gobject_class, PROP_TIME_QUEUED,
       g_param_spec_uint64 ("time-queued", "Time queued",
           "Number of time currently queued", 0, G_MAXUINT64, 0,
-          G_PARAM_READABLE));
+          G_PARAM_READABLE | G_PARAM_STATIC_STRINGS));
 #endif
 
   g_object_class_install_property (gobject_class, PROP_RECOVER_POLICY,
       g_param_spec_enum ("recover-policy", "Recover Policy",
           "How to recover when client reaches the soft max",
-          GST_TYPE_RECOVER_POLICY, DEFAULT_RECOVER_POLICY, G_PARAM_READWRITE));
+          GST_TYPE_RECOVER_POLICY, DEFAULT_RECOVER_POLICY,
+          G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
   g_object_class_install_property (gobject_class, PROP_TIMEOUT,
       g_param_spec_uint64 ("timeout", "Timeout",
           "Maximum inactivity timeout in nanoseconds for a client (0 = no limit)",
-          0, G_MAXUINT64, DEFAULT_TIMEOUT, G_PARAM_READWRITE));
+          0, G_MAXUINT64, DEFAULT_TIMEOUT,
+          G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
   g_object_class_install_property (gobject_class, PROP_SYNC_METHOD,
       g_param_spec_enum ("sync-method", "Sync Method",
-          "How to sync new clients to the stream",
-          GST_TYPE_SYNC_METHOD, DEFAULT_SYNC_METHOD, G_PARAM_READWRITE));
+          "How to sync new clients to the stream", GST_TYPE_SYNC_METHOD,
+          DEFAULT_SYNC_METHOD, G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
   g_object_class_install_property (gobject_class, PROP_BYTES_TO_SERVE,
       g_param_spec_uint64 ("bytes-to-serve", "Bytes to serve",
           "Number of bytes received to serve to clients", 0, G_MAXUINT64, 0,
-          G_PARAM_READABLE));
+          G_PARAM_READABLE | G_PARAM_STATIC_STRINGS));
   g_object_class_install_property (gobject_class, PROP_BYTES_SERVED,
       g_param_spec_uint64 ("bytes-served", "Bytes served",
           "Total number of bytes send to all clients", 0, G_MAXUINT64, 0,
-          G_PARAM_READABLE));
+          G_PARAM_READABLE | G_PARAM_STATIC_STRINGS));
 
   g_object_class_install_property (gobject_class, PROP_BURST_UNIT,
       g_param_spec_enum ("burst-unit", "Burst unit",
           "The format of the burst units (when sync-method is burst[[-with]-keyframe])",
-          GST_TYPE_UNIT_TYPE, DEFAULT_BURST_UNIT, G_PARAM_READWRITE));
+          GST_TYPE_UNIT_TYPE, DEFAULT_BURST_UNIT,
+          G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
   g_object_class_install_property (gobject_class, PROP_BURST_VALUE,
       g_param_spec_uint64 ("burst-value", "Burst value",
-          "The amount of burst expressed in burst-unit",
-          0, G_MAXUINT64, DEFAULT_BURST_VALUE, G_PARAM_READWRITE));
+          "The amount of burst expressed in burst-unit", 0, G_MAXUINT64,
+          DEFAULT_BURST_VALUE, G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
+
+  g_object_class_install_property (gobject_class, PROP_QOS_DSCP,
+      g_param_spec_int ("qos_dscp", "QoS diff srv code point",
+          "Quality of Service, differentiated services code point (-1 default)",
+          -1, 63, DEFAULT_QOS_DSCP, G_PARAM_READWRITE));
 
   /**
    * GstMultiFdSink::add:
@@ -472,9 +510,10 @@ gst_multi_fd_sink_class_init (GstMultiFdSinkClass * klass)
    * Hand the given open file descriptor to multifdsink to write to.
    */
   gst_multi_fd_sink_signals[SIGNAL_ADD] =
-      g_signal_new ("add", G_TYPE_FROM_CLASS (klass), G_SIGNAL_RUN_LAST,
-      G_STRUCT_OFFSET (GstMultiFdSinkClass, add),
-      NULL, NULL, g_cclosure_marshal_VOID__INT, G_TYPE_NONE, 1, G_TYPE_INT);
+      g_signal_new ("add", G_TYPE_FROM_CLASS (klass),
+      G_SIGNAL_RUN_LAST | G_SIGNAL_ACTION, G_STRUCT_OFFSET (GstMultiFdSinkClass,
+          add), NULL, NULL, g_cclosure_marshal_VOID__INT, G_TYPE_NONE, 1,
+      G_TYPE_INT);
   /**
    * GstMultiFdSink::add-full:
    * @gstmultifdsink: the multifdsink element to emit this signal on
@@ -491,11 +530,12 @@ gst_multi_fd_sink_class_init (GstMultiFdSinkClass * klass)
    * specify the burst parameters for the new connection.
    */
   gst_multi_fd_sink_signals[SIGNAL_ADD_BURST] =
-      g_signal_new ("add-full", G_TYPE_FROM_CLASS (klass), G_SIGNAL_RUN_LAST,
-      G_STRUCT_OFFSET (GstMultiFdSinkClass, add_full),
-      NULL, NULL, gst_tcp_marshal_VOID__INT_BOOLEAN_INT_UINT64_INT_UINT64,
-      G_TYPE_NONE, 6, G_TYPE_INT, G_TYPE_BOOLEAN, GST_TYPE_UNIT_TYPE,
-      G_TYPE_UINT64, GST_TYPE_UNIT_TYPE, G_TYPE_UINT64);
+      g_signal_new ("add-full", G_TYPE_FROM_CLASS (klass),
+      G_SIGNAL_RUN_LAST | G_SIGNAL_ACTION, G_STRUCT_OFFSET (GstMultiFdSinkClass,
+          add_full), NULL, NULL,
+      gst_tcp_marshal_VOID__INT_BOOLEAN_INT_UINT64_INT_UINT64, G_TYPE_NONE, 6,
+      G_TYPE_INT, G_TYPE_BOOLEAN, GST_TYPE_UNIT_TYPE, G_TYPE_UINT64,
+      GST_TYPE_UNIT_TYPE, G_TYPE_UINT64);
   /**
    * GstMultiFdSink::remove:
    * @gstmultifdsink: the multifdsink element to emit this signal on
@@ -504,11 +544,12 @@ gst_multi_fd_sink_class_init (GstMultiFdSinkClass * klass)
    * Remove the given open file descriptor from multifdsink.
    */
   gst_multi_fd_sink_signals[SIGNAL_REMOVE] =
-      g_signal_new ("remove", G_TYPE_FROM_CLASS (klass), G_SIGNAL_RUN_LAST,
-      G_STRUCT_OFFSET (GstMultiFdSinkClass, remove),
-      NULL, NULL, gst_tcp_marshal_VOID__INT, G_TYPE_NONE, 1, G_TYPE_INT);
+      g_signal_new ("remove", G_TYPE_FROM_CLASS (klass),
+      G_SIGNAL_RUN_LAST | G_SIGNAL_ACTION, G_STRUCT_OFFSET (GstMultiFdSinkClass,
+          remove), NULL, NULL, gst_tcp_marshal_VOID__INT, G_TYPE_NONE, 1,
+      G_TYPE_INT);
   /**
-   * GstMultiFdSink::remove_flush:
+   * GstMultiFdSink::remove-flush:
    * @gstmultifdsink: the multifdsink element to emit this signal on
    * @fd:             the file descriptor to remove from multifdsink
    *
@@ -517,8 +558,9 @@ gst_multi_fd_sink_class_init (GstMultiFdSinkClass * klass)
    */
   gst_multi_fd_sink_signals[SIGNAL_REMOVE_FLUSH] =
       g_signal_new ("remove-flush", G_TYPE_FROM_CLASS (klass),
-      G_SIGNAL_RUN_LAST, G_STRUCT_OFFSET (GstMultiFdSinkClass, remove_flush),
-      NULL, NULL, gst_tcp_marshal_VOID__INT, G_TYPE_NONE, 1, G_TYPE_INT);
+      G_SIGNAL_RUN_LAST | G_SIGNAL_ACTION, G_STRUCT_OFFSET (GstMultiFdSinkClass,
+          remove_flush), NULL, NULL, gst_tcp_marshal_VOID__INT, G_TYPE_NONE, 1,
+      G_TYPE_INT);
   /**
    * GstMultiFdSink::clear:
    * @gstmultifdsink: the multifdsink element to emit this signal on
@@ -528,9 +570,9 @@ gst_multi_fd_sink_class_init (GstMultiFdSinkClass * klass)
    * should do so by connecting to the client-fd-removed callback.
    */
   gst_multi_fd_sink_signals[SIGNAL_CLEAR] =
-      g_signal_new ("clear", G_TYPE_FROM_CLASS (klass), G_SIGNAL_RUN_LAST,
-      G_STRUCT_OFFSET (GstMultiFdSinkClass, clear),
-      NULL, NULL, g_cclosure_marshal_VOID__VOID, G_TYPE_NONE, 0);
+      g_signal_new ("clear", G_TYPE_FROM_CLASS (klass),
+      G_SIGNAL_RUN_LAST | G_SIGNAL_ACTION, G_STRUCT_OFFSET (GstMultiFdSinkClass,
+          clear), NULL, NULL, g_cclosure_marshal_VOID__VOID, G_TYPE_NONE, 0);
 
   /**
    * GstMultiFdSink::get-stats:
@@ -549,10 +591,10 @@ gst_multi_fd_sink_class_init (GstMultiFdSinkClass * klass)
    *     The array can be 0-length if the client was not found.
    */
   gst_multi_fd_sink_signals[SIGNAL_GET_STATS] =
-      g_signal_new ("get-stats", G_TYPE_FROM_CLASS (klass), G_SIGNAL_RUN_LAST,
-      G_STRUCT_OFFSET (GstMultiFdSinkClass, get_stats),
-      NULL, NULL, gst_tcp_marshal_BOXED__INT, G_TYPE_VALUE_ARRAY, 1,
-      G_TYPE_INT);
+      g_signal_new ("get-stats", G_TYPE_FROM_CLASS (klass),
+      G_SIGNAL_RUN_LAST | G_SIGNAL_ACTION, G_STRUCT_OFFSET (GstMultiFdSinkClass,
+          get_stats), NULL, NULL, gst_tcp_marshal_BOXED__INT,
+      G_TYPE_VALUE_ARRAY, 1, G_TYPE_INT);
 
   /**
    * GstMultiFdSink::client-added:
@@ -648,6 +690,8 @@ gst_multi_fd_sink_init (GstMultiFdSink * this, GstMultiFdSinkClass * klass)
   this->def_burst_unit = DEFAULT_BURST_UNIT;
   this->def_burst_value = DEFAULT_BURST_VALUE;
 
+  this->qos_dscp = DEFAULT_QOS_DSCP;
+
   this->header_flags = 0;
 }
 
@@ -665,11 +709,87 @@ gst_multi_fd_sink_finalize (GObject * object)
   G_OBJECT_CLASS (parent_class)->finalize (object);
 }
 
+static gint
+setup_dscp_client (GstMultiFdSink * sink, GstTCPClient * client)
+{
+  gint tos;
+  gint ret;
+  struct sockaddr_storage ssaddr;
+  socklen_t slen = sizeof (ssaddr);
+  gint af;
+
+  /* don't touch */
+  if (sink->qos_dscp < 0)
+    return 0;
+
+  if ((ret =
+          getsockname (client->fd.fd, (struct sockaddr *) &ssaddr,
+              &slen)) < 0) {
+    GST_DEBUG_OBJECT (sink, "could not get sockname: %s", g_strerror (errno));
+    return ret;
+  }
+
+  af = ssaddr.ss_family;
+
+  /* if this is an IPv4-mapped address then do IPv4 QoS */
+  if (af == AF_INET6) {
+    struct sockaddr_in6 *saddr6 = (struct sockaddr_in6 *) &ssaddr;
+
+    GST_DEBUG_OBJECT (sink, "check IP6 socket");
+    if (IN6_IS_ADDR_V4MAPPED (&(saddr6->sin6_addr))) {
+      GST_DEBUG_OBJECT (sink, "mapped to IPV4");
+      af = AF_INET;
+    }
+  }
+
+  /* extract and shift 6 bits of the DSCP */
+  tos = (sink->qos_dscp & 0x3f) << 2;
+
+  switch (af) {
+    case AF_INET:
+      ret = setsockopt (client->fd.fd, IPPROTO_IP, IP_TOS, &tos, sizeof (tos));
+      break;
+    case AF_INET6:
+#ifdef IPV6_TCLASS
+      ret =
+          setsockopt (client->fd.fd, IPPROTO_IPV6, IPV6_TCLASS, &tos,
+          sizeof (tos));
+      break;
+#endif
+    default:
+      ret = 0;
+      GST_ERROR_OBJECT (sink, "unsupported AF");
+      break;
+  }
+  if (ret)
+    GST_DEBUG_OBJECT (sink, "could not set DSCP: %s", g_strerror (errno));
+
+  return ret;
+}
+
+
+static void
+setup_dscp (GstMultiFdSink * sink)
+{
+  GList *clients, *next;
+
+  CLIENTS_LOCK (sink);
+  for (clients = sink->clients; clients; clients = next) {
+    GstTCPClient *client;
+
+    client = (GstTCPClient *) clients->data;
+    next = g_list_next (clients);
+
+    setup_dscp_client (sink, client);
+  }
+  CLIENTS_UNLOCK (sink);
+}
+
 /* "add-full" signal implementation */
 void
 gst_multi_fd_sink_add_full (GstMultiFdSink * sink, int fd,
-    GstSyncMethod sync_method, GstUnitType min_unit, guint64 min_value,
-    GstUnitType max_unit, guint64 max_value)
+    GstSyncMethod sync_method, GstTCPUnitType min_unit, guint64 min_value,
+    GstTCPUnitType max_unit, guint64 max_value)
 {
   GstTCPClient *client;
   GList *clink;
@@ -729,20 +849,21 @@ gst_multi_fd_sink_add_full (GstMultiFdSink * sink, int fd,
   /* set the socket to non blocking */
   res = fcntl (fd, F_SETFL, O_NONBLOCK);
   /* we always read from a client */
-  gst_fdset_add_fd (sink->fdset, &client->fd);
+  gst_poll_add_fd (sink->fdset, &client->fd);
 
   /* we don't try to read from write only fds */
   flags = fcntl (fd, F_GETFL, 0);
   if ((flags & O_ACCMODE) != O_WRONLY) {
-    gst_fdset_fd_ctl_read (sink->fdset, &client->fd, TRUE);
+    gst_poll_fd_ctl_read (sink->fdset, &client->fd, TRUE);
   }
   /* figure out the mode, can't use send() for non sockets */
   res = fstat (fd, &statbuf);
   if (S_ISSOCK (statbuf.st_mode)) {
     client->is_socket = TRUE;
+    setup_dscp_client (sink, client);
   }
 
-  SEND_COMMAND (sink, CONTROL_RESTART);
+  gst_poll_restart (sink->fdset);
 
   CLIENTS_UNLOCK (sink);
 
@@ -803,7 +924,7 @@ gst_multi_fd_sink_remove (GstMultiFdSink * sink, int fd)
 
     client->status = GST_CLIENT_STATUS_REMOVED;
     gst_multi_fd_sink_remove_client_link (sink, clink);
-    SEND_COMMAND (sink, CONTROL_RESTART);
+    gst_poll_restart (sink->fdset);
   } else {
     GST_WARNING_OBJECT (sink, "[fd %5d] no client with this fd found!", fd);
   }
@@ -873,7 +994,7 @@ restart:
     client->status = GST_CLIENT_STATUS_REMOVED;
     gst_multi_fd_sink_remove_client_link (sink, clients);
   }
-  SEND_COMMAND (sink, CONTROL_RESTART);
+  gst_poll_restart (sink->fdset);
   CLIENTS_UNLOCK (sink);
 }
 
@@ -1006,7 +1127,7 @@ gst_multi_fd_sink_remove_client_link (GstMultiFdSink * sink, GList * link)
       break;
   }
 
-  gst_fdset_remove_fd (sink->fdset, &client->fd);
+  gst_poll_remove_fd (sink->fdset, &client->fd);
 
   g_get_current_time (&now);
   client->disconnect_time = GST_TIMEVAL_TO_TIME (now);
@@ -1374,9 +1495,9 @@ static gint
 get_buffers_max (GstMultiFdSink * sink, gint64 max)
 {
   switch (sink->unit_type) {
-    case GST_UNIT_TYPE_BUFFERS:
+    case GST_TCP_UNIT_TYPE_BUFFERS:
       return max;
-    case GST_UNIT_TYPE_TIME:
+    case GST_TCP_UNIT_TYPE_TIME:
     {
       GstBuffer *buf;
       int i;
@@ -1400,7 +1521,7 @@ get_buffers_max (GstMultiFdSink * sink, gint64 max)
       }
       return len + 1;
     }
-    case GST_UNIT_TYPE_BYTES:
+    case GST_TCP_UNIT_TYPE_BYTES:
     {
       GstBuffer *buf;
       int i;
@@ -1543,23 +1664,23 @@ find_limits (GstMultiFdSink * sink,
  * Returns: FALSE if the unit is unknown or undefined. TRUE otherwise.
  */
 static gboolean
-assign_value (GstUnitType unit, guint64 value, gint * bytes, gint * buffers,
+assign_value (GstTCPUnitType unit, guint64 value, gint * bytes, gint * buffers,
     GstClockTime * time)
 {
   gboolean res = TRUE;
 
   /* set only the limit of the given format to the given value */
   switch (unit) {
-    case GST_UNIT_TYPE_BUFFERS:
+    case GST_TCP_UNIT_TYPE_BUFFERS:
       *buffers = (gint) value;
       break;
-    case GST_UNIT_TYPE_TIME:
+    case GST_TCP_UNIT_TYPE_TIME:
       *time = value;
       break;
-    case GST_UNIT_TYPE_BYTES:
+    case GST_TCP_UNIT_TYPE_BYTES:
       *bytes = (gint) value;
       break;
-    case GST_UNIT_TYPE_UNDEFINED:
+    case GST_TCP_UNIT_TYPE_UNDEFINED:
     default:
       res = FALSE;
       break;
@@ -1576,8 +1697,9 @@ assign_value (GstUnitType unit, guint64 value, gint * bytes, gint * buffers,
  * function returns FALSE.
  */
 static gboolean
-count_burst_unit (GstMultiFdSink * sink, gint * min_idx, GstUnitType min_unit,
-    guint64 min_value, gint * max_idx, GstUnitType max_unit, guint64 max_value)
+count_burst_unit (GstMultiFdSink * sink, gint * min_idx,
+    GstTCPUnitType min_unit, guint64 min_value, gint * max_idx,
+    GstTCPUnitType max_unit, guint64 max_value)
 {
   gint bytes_min = -1, buffers_min = -1;
   gint bytes_max = -1, buffers_max = -1;
@@ -1873,7 +1995,7 @@ gst_multi_fd_sink_handle_client_write (GstMultiFdSink * sink,
       if (client->bufpos == -1) {
         /* client is too fast, remove from write queue until new buffer is
          * available */
-        gst_fdset_fd_ctl_write (sink->fdset, &client->fd, FALSE);
+        gst_poll_fd_ctl_write (sink->fdset, &client->fd, FALSE);
         /* if we flushed out all of the client buffers, we can stop */
         if (client->flushcount == 0)
           goto flushed;
@@ -1894,7 +2016,7 @@ gst_multi_fd_sink_handle_client_write (GstMultiFdSink * sink,
             client->bufpos = position;
           } else {
             /* cannot send data to this client yet */
-            gst_fdset_fd_ctl_write (sink->fdset, &client->fd, FALSE);
+            gst_poll_fd_ctl_write (sink->fdset, &client->fd, FALSE);
             return TRUE;
           }
         }
@@ -2160,7 +2282,7 @@ restart:
     } else if (client->bufpos == 0 || client->new_connection) {
       /* can send data to this client now. need to signal the select thread that
        * the fd_set changed */
-      gst_fdset_fd_ctl_write (sink->fdset, &client->fd, TRUE);
+      gst_poll_fd_ctl_write (sink->fdset, &client->fd, TRUE);
       need_signal = TRUE;
     }
     /* keep track of maximum buffer usage */
@@ -2237,7 +2359,7 @@ restart:
 
   /* and send a signal to thread if fd_set changed */
   if (need_signal) {
-    SEND_COMMAND (sink, CONTROL_RESTART);
+    gst_poll_restart (sink->fdset);
   }
 }
 
@@ -2261,8 +2383,6 @@ gst_multi_fd_sink_handle_clients (GstMultiFdSink * sink)
   fclass = GST_MULTI_FD_SINK_GET_CLASS (sink);
 
   do {
-    gboolean stop = FALSE;
-
     try_again = FALSE;
 
     /* check for:
@@ -2270,7 +2390,7 @@ gst_multi_fd_sink_handle_clients (GstMultiFdSink * sink)
      * - client socket input (ie, clients saying goodbye)
      * - client socket output (ie, client reads)          */
     GST_LOG_OBJECT (sink, "waiting on action on fdset");
-    result = gst_fdset_wait (sink->fdset, -1);
+    result = gst_poll_wait (sink->fdset, GST_CLOCK_TIME_NONE);
 
     /* < 0 is an error, 0 just means a timeout happened, which is impossible */
     if (result < 0) {
@@ -2313,8 +2433,11 @@ gst_multi_fd_sink_handle_clients (GstMultiFdSink * sink)
          * are not valid */
         try_again = TRUE;
       } else if (errno == EINTR) {
-        /* interrupted system call, just redo the select */
+        /* interrupted system call, just redo the wait */
         try_again = TRUE;
+      } else if (errno == EBUSY) {
+        /* the call to gst_poll_wait() was flushed */
+        return;
       } else {
         /* this is quite bad... */
         GST_ELEMENT_ERROR (sink, RESOURCE, READ, (NULL),
@@ -2323,46 +2446,6 @@ gst_multi_fd_sink_handle_clients (GstMultiFdSink * sink)
       }
     } else {
       GST_LOG_OBJECT (sink, "wait done: %d sockets with events", result);
-      /* read all commands */
-      if (gst_fdset_fd_can_read (sink->fdset, &READ_SOCKET (sink))) {
-        GST_LOG_OBJECT (sink, "have a command");
-        while (TRUE) {
-          gchar command;
-          int res;
-
-          READ_COMMAND (sink, command, res);
-          if (res <= 0) {
-            GST_LOG_OBJECT (sink, "no more commands");
-            /* no more commands */
-            break;
-          }
-
-          switch (command) {
-            case CONTROL_RESTART:
-              GST_LOG_OBJECT (sink, "restart");
-              /* need to restart the select call as the fd_set changed */
-              /* if other file descriptors than the READ_SOCKET had activity,
-               * we don't restart just yet, but handle the other clients first
-               */
-              if (result == 1)
-                try_again = TRUE;
-              break;
-            case CONTROL_STOP:
-              /* break out of the select loop */
-              GST_LOG_OBJECT (sink, "stop");
-              /* stop this function */
-              stop = TRUE;
-              break;
-            default:
-              GST_WARNING_OBJECT (sink, "unkown");
-              g_warning ("multifdsink: unknown control message received");
-              break;
-          }
-        }
-      }
-    }
-    if (stop) {
-      return;
     }
   } while (try_again);
 
@@ -2392,25 +2475,25 @@ restart2:
       continue;
     }
 
-    if (gst_fdset_fd_has_closed (sink->fdset, &client->fd)) {
+    if (gst_poll_fd_has_closed (sink->fdset, &client->fd)) {
       client->status = GST_CLIENT_STATUS_CLOSED;
       gst_multi_fd_sink_remove_client_link (sink, clients);
       continue;
     }
-    if (gst_fdset_fd_has_error (sink->fdset, &client->fd)) {
-      GST_WARNING_OBJECT (sink, "gst_fdset_fd_has_error for %d", client->fd.fd);
+    if (gst_poll_fd_has_error (sink->fdset, &client->fd)) {
+      GST_WARNING_OBJECT (sink, "gst_poll_fd_has_error for %d", client->fd.fd);
       client->status = GST_CLIENT_STATUS_ERROR;
       gst_multi_fd_sink_remove_client_link (sink, clients);
       continue;
     }
-    if (gst_fdset_fd_can_read (sink->fdset, &client->fd)) {
+    if (gst_poll_fd_can_read (sink->fdset, &client->fd)) {
       /* handle client read */
       if (!gst_multi_fd_sink_handle_client_read (sink, client)) {
         gst_multi_fd_sink_remove_client_link (sink, clients);
         continue;
       }
     }
-    if (gst_fdset_fd_can_write (sink->fdset, &client->fd)) {
+    if (gst_poll_fd_can_write (sink->fdset, &client->fd)) {
       /* handle client write */
       if (!gst_multi_fd_sink_handle_client_write (sink, client)) {
         gst_multi_fd_sink_remove_client_link (sink, clients);
@@ -2586,6 +2669,10 @@ gst_multi_fd_sink_set_property (GObject * object, guint prop_id,
     case PROP_BURST_VALUE:
       multifdsink->def_burst_value = g_value_get_uint64 (value);
       break;
+    case PROP_QOS_DSCP:
+      multifdsink->qos_dscp = g_value_get_int (value);
+      setup_dscp (multifdsink);
+      break;
 
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
@@ -2662,6 +2749,9 @@ gst_multi_fd_sink_get_property (GObject * object, guint prop_id, GValue * value,
     case PROP_BURST_VALUE:
       g_value_set_uint64 (value, multifdsink->def_burst_value);
       break;
+    case PROP_QOS_DSCP:
+      g_value_set_int (value, multifdsink->qos_dscp);
+      break;
 
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
@@ -2675,7 +2765,6 @@ static gboolean
 gst_multi_fd_sink_start (GstBaseSink * bsink)
 {
   GstMultiFdSinkClass *fclass;
-  int control_socket[2];
   GstMultiFdSink *this;
 
   if (GST_OBJECT_FLAG_IS_SET (bsink, GST_MULTI_FD_SINK_OPEN))
@@ -2685,20 +2774,9 @@ gst_multi_fd_sink_start (GstBaseSink * bsink)
   fclass = GST_MULTI_FD_SINK_GET_CLASS (this);
 
   GST_INFO_OBJECT (this, "starting in mode %d", this->mode);
-  this->fdset = gst_fdset_new (this->mode);
-
-  if (socketpair (PF_UNIX, SOCK_STREAM, 0, control_socket) < 0)
+  if ((this->fdset = gst_poll_new (TRUE)) == NULL)
     goto socket_pair;
 
-  READ_SOCKET (this).fd = control_socket[0];
-  WRITE_SOCKET (this).fd = control_socket[1];
-
-  gst_fdset_add_fd (this->fdset, &READ_SOCKET (this));
-  gst_fdset_fd_ctl_read (this->fdset, &READ_SOCKET (this), TRUE);
-
-  fcntl (READ_SOCKET (this).fd, F_SETFL, O_NONBLOCK);
-  fcntl (WRITE_SOCKET (this).fd, F_SETFL, O_NONBLOCK);
-
   this->streamheader = NULL;
   this->bytes_to_serve = 0;
   this->bytes_served = 0;
@@ -2746,7 +2824,7 @@ gst_multi_fd_sink_stop (GstBaseSink * bsink)
 
   this->running = FALSE;
 
-  SEND_COMMAND (this, CONTROL_STOP);
+  gst_poll_set_flushing (this->fdset, TRUE);
   if (this->thread) {
     GST_DEBUG_OBJECT (this, "joining thread");
     g_thread_join (this->thread);
@@ -2757,9 +2835,6 @@ gst_multi_fd_sink_stop (GstBaseSink * bsink)
   /* free the clients */
   gst_multi_fd_sink_clear (this);
 
-  close (READ_SOCKET (this).fd);
-  close (WRITE_SOCKET (this).fd);
-
   if (this->streamheader) {
     g_slist_foreach (this->streamheader, (GFunc) gst_mini_object_unref, NULL);
     g_slist_free (this->streamheader);
@@ -2770,8 +2845,7 @@ gst_multi_fd_sink_stop (GstBaseSink * bsink)
     fclass->close (this);
 
   if (this->fdset) {
-    gst_fdset_remove_fd (this->fdset, &READ_SOCKET (this));
-    gst_fdset_free (this->fdset);
+    gst_poll_free (this->fdset);
     this->fdset = NULL;
   }
   g_hash_table_foreach_remove (this->fd_hash, multifdsink_hash_remove, this);
