@@ -80,7 +80,7 @@ GST_ELEMENT_DETAILS ("Video scaler",
     "Resizes video",
     "Wim Taymans <wim.taymans@chello.be>");
 
-#define DEFAULT_PROP_METHOD	GST_VIDEO_SCALE_NEAREST
+#define DEFAULT_PROP_METHOD	GST_VIDEO_SCALE_BILINEAR
 
 enum
 {
@@ -139,6 +139,7 @@ static GType
 gst_video_scale_method_get_type (void)
 {
   static GType video_scale_method_type = 0;
+
   static const GEnumValue video_scale_methods[] = {
     {GST_VIDEO_SCALE_NEAREST, "Nearest Neighbour", "nearest-neighbour"},
     {GST_VIDEO_SCALE_BILINEAR, "Bilinear", "bilinear"},
@@ -187,9 +188,13 @@ gst_video_scale_sink_template_factory (void)
 
 
 static void gst_video_scale_base_init (gpointer g_class);
+
 static void gst_video_scale_class_init (GstVideoScaleClass * klass);
+
 static void gst_video_scale_init (GstVideoScale * videoscale);
+
 static void gst_video_scale_finalize (GstVideoScale * videoscale);
+
 static gboolean gst_video_scale_src_event (GstBaseTransform * trans,
     GstEvent * event);
 
@@ -255,6 +260,7 @@ static void
 gst_video_scale_class_init (GstVideoScaleClass * klass)
 {
   GObjectClass *gobject_class;
+
   GstBaseTransformClass *trans_class;
 
   gobject_class = (GObjectClass *) klass;
@@ -266,7 +272,8 @@ gst_video_scale_class_init (GstVideoScaleClass * klass)
 
   g_object_class_install_property (gobject_class, PROP_METHOD,
       g_param_spec_enum ("method", "method", "method",
-          GST_TYPE_VIDEO_SCALE_METHOD, DEFAULT_PROP_METHOD, G_PARAM_READWRITE));
+          GST_TYPE_VIDEO_SCALE_METHOD, DEFAULT_PROP_METHOD,
+          G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
 
   trans_class->transform_caps =
       GST_DEBUG_FUNCPTR (gst_video_scale_transform_caps);
@@ -303,11 +310,13 @@ static void
 gst_video_scale_set_property (GObject * object, guint prop_id,
     const GValue * value, GParamSpec * pspec)
 {
-  GstVideoScale *src = GST_VIDEO_SCALE (object);
+  GstVideoScale *vscale = GST_VIDEO_SCALE (object);
 
   switch (prop_id) {
     case PROP_METHOD:
-      src->method = g_value_get_enum (value);
+      GST_OBJECT_LOCK (vscale);
+      vscale->method = g_value_get_enum (value);
+      GST_OBJECT_UNLOCK (vscale);
       break;
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
@@ -319,11 +328,13 @@ static void
 gst_video_scale_get_property (GObject * object, guint prop_id, GValue * value,
     GParamSpec * pspec)
 {
-  GstVideoScale *src = GST_VIDEO_SCALE (object);
+  GstVideoScale *vscale = GST_VIDEO_SCALE (object);
 
   switch (prop_id) {
     case PROP_METHOD:
-      g_value_set_enum (value, src->method);
+      GST_OBJECT_LOCK (vscale);
+      g_value_set_enum (value, vscale->method);
+      GST_OBJECT_UNLOCK (vscale);
       break;
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
@@ -336,50 +347,75 @@ gst_video_scale_transform_caps (GstBaseTransform * trans,
     GstPadDirection direction, GstCaps * caps)
 {
   GstVideoScale *videoscale;
+
   GstCaps *ret;
+
   GstStructure *structure;
+
   const GValue *par;
 
+  gint method;
+
+  /* this function is always called with a simple caps */
+  g_return_val_if_fail (GST_CAPS_IS_SIMPLE (caps), NULL);
+
   videoscale = GST_VIDEO_SCALE (trans);
 
+  GST_OBJECT_LOCK (videoscale);
+  method = videoscale->method;
+  GST_OBJECT_UNLOCK (videoscale);
+
+  structure = gst_caps_get_structure (caps, 0);
+
+  /* check compatibility of format and method before we copy the input caps */
+  if (method == GST_VIDEO_SCALE_4TAP) {
+    guint32 fourcc;
+
+    if (!gst_structure_has_name (structure, "video/x-raw-yuv"))
+      goto method_not_implemented_for_format;
+    if (!gst_structure_get_fourcc (structure, "format", &fourcc))
+      goto method_not_implemented_for_format;
+    if (fourcc != GST_MAKE_FOURCC ('I', '4', '2', '0') &&
+        fourcc != GST_MAKE_FOURCC ('Y', 'V', '1', '2'))
+      goto method_not_implemented_for_format;
+  }
+
   ret = gst_caps_copy (caps);
-
-  /* this function is always called with a simple caps */
-  g_return_val_if_fail (GST_CAPS_IS_SIMPLE (ret), NULL);
-
-  structure = gst_caps_get_structure (ret, 0);
+  structure = gst_structure_copy (gst_caps_get_structure (ret, 0));
 
   gst_structure_set (structure,
       "width", GST_TYPE_INT_RANGE, 1, G_MAXINT,
       "height", GST_TYPE_INT_RANGE, 1, G_MAXINT, NULL);
 
+  gst_caps_merge_structure (ret, gst_structure_copy (structure));
+
   /* if pixel aspect ratio, make a range of it */
   if ((par = gst_structure_get_value (structure, "pixel-aspect-ratio"))) {
-    GstCaps *copy;
-    GstStructure *cstruct;
-
-    /* copy input PAR first, this is the prefered PAR */
-    gst_structure_set_value (structure, "pixel-aspect-ratio", par);
-
-    /* then make a copy with a fraction range as a second choice */
-    copy = gst_caps_copy (ret);
-    cstruct = gst_caps_get_structure (copy, 0);
-    gst_structure_set (cstruct,
+    gst_structure_set (structure,
         "pixel-aspect-ratio", GST_TYPE_FRACTION_RANGE, 0, 1, G_MAXINT, 1, NULL);
 
-    /* and append */
-    gst_caps_append (ret, copy);
+    gst_caps_merge_structure (ret, structure);
+  } else {
+    gst_structure_free (structure);
   }
 
   GST_DEBUG_OBJECT (trans, "returning caps: %" GST_PTR_FORMAT, ret);
 
   return ret;
+
+method_not_implemented_for_format:
+  {
+    GST_DEBUG_OBJECT (trans, "method %d not implemented for format %"
+        GST_PTR_FORMAT ", returning empty caps", method, caps);
+    return gst_caps_new_empty ();
+  }
 }
 
 static int
 gst_video_scale_get_format (GstCaps * caps)
 {
   int i;
+
   GstCaps *icaps, *scaps;
 
   for (i = 0; i < G_N_ELEMENTS (gst_video_scale_format_caps); i++) {
@@ -474,6 +510,7 @@ static gboolean
 parse_caps (GstCaps * caps, gint * format, gint * width, gint * height)
 {
   gboolean ret;
+
   GstStructure *structure;
 
   structure = gst_caps_get_structure (caps, 0);
@@ -490,6 +527,7 @@ static gboolean
 gst_video_scale_set_caps (GstBaseTransform * trans, GstCaps * in, GstCaps * out)
 {
   GstVideoScale *videoscale;
+
   gboolean ret;
 
   videoscale = GST_VIDEO_SCALE (trans);
@@ -531,7 +569,9 @@ gst_video_scale_get_unit_size (GstBaseTransform * trans, GstCaps * caps,
     guint * size)
 {
   GstVideoScale *videoscale;
+
   gint format, width, height;
+
   VSImage img;
 
   g_assert (size);
@@ -553,6 +593,7 @@ gst_video_scale_fixate_caps (GstBaseTransform * base, GstPadDirection direction,
     GstCaps * caps, GstCaps * othercaps)
 {
   GstStructure *ins, *outs;
+
   const GValue *from_par, *to_par;
 
   g_return_if_fail (gst_caps_is_fixed (caps));
@@ -569,7 +610,9 @@ gst_video_scale_fixate_caps (GstBaseTransform * base, GstPadDirection direction,
   /* we have both PAR but they might not be fixated */
   if (from_par && to_par) {
     gint from_w, from_h, from_par_n, from_par_d, to_par_n, to_par_d;
+
     gint count = 0, w = 0, h = 0;
+
     guint num, den;
 
     /* from_par should be fixed */
@@ -698,17 +741,28 @@ gst_video_scale_transform (GstBaseTransform * trans, GstBuffer * in,
     GstBuffer * out)
 {
   GstVideoScale *videoscale;
+
   GstFlowReturn ret = GST_FLOW_OK;
+
   VSImage *dest;
+
   VSImage *src;
+
   VSImage dest_u;
+
   VSImage dest_v;
+
   VSImage src_u;
+
   VSImage src_v;
 
+  gint method;
+
   videoscale = GST_VIDEO_SCALE (trans);
 
-  gst_buffer_stamp (out, in);
+  GST_OBJECT_LOCK (videoscale);
+  method = videoscale->method;
+  GST_OBJECT_UNLOCK (videoscale);
 
   src = &videoscale->src;
   dest = &videoscale->dest;
@@ -717,8 +771,9 @@ gst_video_scale_transform (GstBaseTransform * trans, GstBuffer * in,
   gst_video_scale_prepare_image (videoscale->format, out, dest, &dest_u,
       &dest_v);
 
-  switch (videoscale->method) {
+  switch (method) {
     case GST_VIDEO_SCALE_NEAREST:
+      GST_LOG_OBJECT (videoscale, "doing nearest scaling");
       switch (videoscale->format) {
         case GST_VIDEO_SCALE_RGBx:
         case GST_VIDEO_SCALE_xRGB:
@@ -762,6 +817,7 @@ gst_video_scale_transform (GstBaseTransform * trans, GstBuffer * in,
       }
       break;
     case GST_VIDEO_SCALE_BILINEAR:
+      GST_LOG_OBJECT (videoscale, "doing bilinear scaling");
       switch (videoscale->format) {
         case GST_VIDEO_SCALE_RGBx:
         case GST_VIDEO_SCALE_xRGB:
@@ -805,6 +861,7 @@ gst_video_scale_transform (GstBaseTransform * trans, GstBuffer * in,
       }
       break;
     case GST_VIDEO_SCALE_4TAP:
+      GST_LOG_OBJECT (videoscale, "doing 4tap scaling");
       switch (videoscale->format) {
         case GST_VIDEO_SCALE_I420:
         case GST_VIDEO_SCALE_YV12:
@@ -813,6 +870,8 @@ gst_video_scale_transform (GstBaseTransform * trans, GstBuffer * in,
           vs_image_scale_4tap_Y (&dest_v, &src_v, videoscale->tmp_buf);
           break;
         default:
+          /* FIXME: update gst_video_scale_transform_caps once RGB and/or
+           * other YUV formats work too */
           goto unsupported;
       }
       break;
@@ -830,7 +889,7 @@ unsupported:
   {
     GST_ELEMENT_ERROR (videoscale, STREAM, NOT_IMPLEMENTED, (NULL),
         ("Unsupported format %d for scaling method %d",
-            videoscale->format, videoscale->method));
+            videoscale->format, method));
     return GST_FLOW_ERROR;
   }
 unknown_mode:
@@ -845,8 +904,11 @@ static gboolean
 gst_video_scale_src_event (GstBaseTransform * trans, GstEvent * event)
 {
   GstVideoScale *videoscale;
+
   gboolean ret;
+
   double a;
+
   GstStructure *structure;
 
   videoscale = GST_VIDEO_SCALE (trans);
