@@ -22,12 +22,12 @@
  * SECTION:element-videotestsrc
  *
  * <refsect2>
- * <title>Example launch line</title>
  * <para>
  * The videotestsrc element is used to produce test video data in a wide variaty
  * of formats. The video test data produced can be controlled with the "pattern"
  * property.
  * </para>
+ * <title>Example launch line</title>
  * <para>
  * <programlisting>
  * gst-launch -v videotestsrc pattern=snow ! ximagesink
@@ -113,6 +113,8 @@ gst_video_test_src_pattern_get_type (void)
     {GST_VIDEO_TEST_SRC_CHECKERS2, "Checkers 2px", "checkers-2"},
     {GST_VIDEO_TEST_SRC_CHECKERS4, "Checkers 4px", "checkers-4"},
     {GST_VIDEO_TEST_SRC_CHECKERS8, "Checkers 8px", "checkers-8"},
+    {GST_VIDEO_TEST_SRC_CIRCULAR, "Circular", "circular"},
+    {GST_VIDEO_TEST_SRC_BLINK, "Blink", "blink"},
     {0, NULL, NULL}
   };
 
@@ -152,15 +154,16 @@ gst_video_test_src_class_init (GstVideoTestSrcClass * klass)
   g_object_class_install_property (gobject_class, PROP_PATTERN,
       g_param_spec_enum ("pattern", "Pattern",
           "Type of test pattern to generate", GST_TYPE_VIDEO_TEST_SRC_PATTERN,
-          1, G_PARAM_READWRITE));
-  g_object_class_install_property (gobject_class,
-      PROP_TIMESTAMP_OFFSET, g_param_spec_int64 ("timestamp-offset",
-          "Timestamp offset",
+          GST_VIDEO_TEST_SRC_SMPTE,
+          G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
+  g_object_class_install_property (gobject_class, PROP_TIMESTAMP_OFFSET,
+      g_param_spec_int64 ("timestamp-offset", "Timestamp offset",
           "An offset added to timestamps set on buffers (in ns)", G_MININT64,
-          G_MAXINT64, 0, G_PARAM_READWRITE));
+          G_MAXINT64, 0, G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
   g_object_class_install_property (gobject_class, PROP_IS_LIVE,
       g_param_spec_boolean ("is-live", "Is Live",
-          "Whether to act as a live source", FALSE, G_PARAM_READWRITE));
+          "Whether to act as a live source", FALSE,
+          G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
 
   gstbasesrc_class->get_caps = gst_video_test_src_getcaps;
   gstbasesrc_class->set_caps = gst_video_test_src_setcaps;
@@ -243,6 +246,12 @@ gst_video_test_src_set_pattern (GstVideoTestSrc * videotestsrc,
     case GST_VIDEO_TEST_SRC_CHECKERS8:
       videotestsrc->make_image = gst_video_test_src_checkers8;
       break;
+    case GST_VIDEO_TEST_SRC_CIRCULAR:
+      videotestsrc->make_image = gst_video_test_src_circular;
+      break;
+    case GST_VIDEO_TEST_SRC_BLINK:
+      videotestsrc->make_image = gst_video_test_src_black;
+      break;
     default:
       g_assert_not_reached ();
   }
@@ -493,7 +502,8 @@ gst_video_test_src_do_seek (GstBaseSrc * bsrc, GstSegment * segment)
 
   src = GST_VIDEO_TEST_SRC (bsrc);
 
-  time = segment->time = segment->start;
+  segment->time = segment->start;
+  time = segment->last_stop;
 
   /* now move to the time indicated */
   if (src->rate_numerator) {
@@ -560,8 +570,18 @@ gst_video_test_src_create (GstPushSrc * psrc, GstBuffer ** buffer)
   gst_buffer_set_caps (outbuf, GST_PAD_CAPS (GST_BASE_SRC_PAD (psrc)));
 #endif
 
-  src->make_image (src, (void *) GST_BUFFER_DATA (outbuf),
-      src->width, src->height);
+  if (src->pattern_type == GST_VIDEO_TEST_SRC_BLINK) {
+    if (src->n_frames & 0x1) {
+      gst_video_test_src_white (src, (void *) GST_BUFFER_DATA (outbuf),
+          src->width, src->height);
+    } else {
+      gst_video_test_src_black (src, (void *) GST_BUFFER_DATA (outbuf),
+          src->width, src->height);
+    }
+  } else {
+    src->make_image (src, (void *) GST_BUFFER_DATA (outbuf),
+        src->width, src->height);
+  }
 
   GST_BUFFER_TIMESTAMP (outbuf) = src->timestamp_offset + src->running_time;
   GST_BUFFER_OFFSET (outbuf) = src->n_frames;
