@@ -144,8 +144,9 @@ GST_START_TEST (test_rtp_buffer_set_extension_data)
   data = GST_BUFFER_DATA (buf);
 
   /* should be impossible to set the extension data */
-  fail_unless (gst_rtp_buffer_set_extension_data (buf, 0, 4) == FALSE);
-  fail_unless (gst_rtp_buffer_get_extension (buf) == TRUE);
+  ASSERT_WARNING (fail_unless (gst_rtp_buffer_set_extension_data (buf, 0,
+              4) == FALSE));
+  fail_unless (gst_rtp_buffer_get_extension (buf) == FALSE);
 
   /* should be possible to set the extension data */
   fail_unless (gst_rtp_buffer_set_extension_data (buf, 270, 0) == TRUE);
@@ -174,6 +175,92 @@ GST_START_TEST (test_rtp_buffer_set_extension_data)
 
 GST_END_TEST;
 
+GST_START_TEST (test_rtp_seqnum_compare)
+{
+#define ASSERT_COMP(a,b,c) fail_unless (gst_rtp_buffer_compare_seqnum ((guint16)a,(guint16)b) == c);
+  ASSERT_COMP (0xfffe, 0xfffd, -1);
+  ASSERT_COMP (0xffff, 0xfffe, -1);
+  ASSERT_COMP (0x0000, 0xffff, -1);
+  ASSERT_COMP (0x0001, 0x0000, -1);
+  ASSERT_COMP (0x0002, 0x0001, -1);
+
+  ASSERT_COMP (0xffff, 0xfffd, -2);
+  ASSERT_COMP (0x0000, 0xfffd, -3);
+  ASSERT_COMP (0x0001, 0xfffd, -4);
+  ASSERT_COMP (0x0002, 0xfffd, -5);
+
+  ASSERT_COMP (0x7ffe, 0x7ffd, -1);
+  ASSERT_COMP (0x7fff, 0x7ffe, -1);
+  ASSERT_COMP (0x8000, 0x7fff, -1);
+  ASSERT_COMP (0x8001, 0x8000, -1);
+  ASSERT_COMP (0x8002, 0x8001, -1);
+
+  ASSERT_COMP (0x7fff, 0x7ffd, -2);
+  ASSERT_COMP (0x8000, 0x7ffd, -3);
+  ASSERT_COMP (0x8001, 0x7ffd, -4);
+  ASSERT_COMP (0x8002, 0x7ffd, -5);
+
+  ASSERT_COMP (0x7ffd, 0xffff, -0x7ffe);
+  ASSERT_COMP (0x7ffe, 0x0000, -0x7ffe);
+  ASSERT_COMP (0x7fff, 0x0001, -0x7ffe);
+  ASSERT_COMP (0x7fff, 0x0000, -0x7fff);
+  ASSERT_COMP (0x8000, 0x0001, -0x7fff);
+  ASSERT_COMP (0x8001, 0x0002, -0x7fff);
+
+  ASSERT_COMP (0xfffd, 0x7ffe, -0x7fff);
+  ASSERT_COMP (0xfffe, 0x7fff, -0x7fff);
+  ASSERT_COMP (0xffff, 0x8000, -0x7fff);
+  ASSERT_COMP (0x0000, 0x8001, -0x7fff);
+  ASSERT_COMP (0x0001, 0x8002, -0x7fff);
+
+  ASSERT_COMP (0xfffe, 0x7ffe, -0x8000);
+  ASSERT_COMP (0xffff, 0x7fff, -0x8000);
+  ASSERT_COMP (0x0000, 0x8000, -0x8000);
+  ASSERT_COMP (0x0001, 0x8001, -0x8000);
+
+  ASSERT_COMP (0x7ffe, 0xfffe, -0x8000);
+  ASSERT_COMP (0x7fff, 0xffff, -0x8000);
+  ASSERT_COMP (0x8000, 0x0000, -0x8000);
+  ASSERT_COMP (0x8001, 0x0001, -0x8000);
+
+  ASSERT_COMP (0x0001, 0x0002, 1);
+  ASSERT_COMP (0x0000, 0x0001, 1);
+  ASSERT_COMP (0xffff, 0x0000, 1);
+  ASSERT_COMP (0xfffe, 0xffff, 1);
+  ASSERT_COMP (0xfffd, 0xfffe, 1);
+
+  ASSERT_COMP (0x0000, 0x0002, 2);
+  ASSERT_COMP (0xffff, 0x0002, 3);
+  ASSERT_COMP (0xfffe, 0x0002, 4);
+  ASSERT_COMP (0xfffd, 0x0002, 5);
+
+  ASSERT_COMP (0x8001, 0x8002, 1);
+  ASSERT_COMP (0x8000, 0x8001, 1);
+  ASSERT_COMP (0x7fff, 0x8000, 1);
+  ASSERT_COMP (0x7ffe, 0x7fff, 1);
+  ASSERT_COMP (0x7ffd, 0x7ffe, 1);
+
+  ASSERT_COMP (0x8000, 0x8002, 2);
+  ASSERT_COMP (0x7fff, 0x8002, 3);
+  ASSERT_COMP (0x7ffe, 0x8002, 4);
+  ASSERT_COMP (0x7ffd, 0x8002, 5);
+
+  ASSERT_COMP (0xfffe, 0x7ffd, 0x7fff);
+  ASSERT_COMP (0xffff, 0x7ffe, 0x7fff);
+  ASSERT_COMP (0x0000, 0x7fff, 0x7fff);
+  ASSERT_COMP (0x0001, 0x8000, 0x7fff);
+  ASSERT_COMP (0x0002, 0x8001, 0x7fff);
+
+  ASSERT_COMP (0x7ffe, 0xfffd, 0x7fff);
+  ASSERT_COMP (0x7fff, 0xfffe, 0x7fff);
+  ASSERT_COMP (0x8000, 0xffff, 0x7fff);
+  ASSERT_COMP (0x8001, 0x0000, 0x7fff);
+  ASSERT_COMP (0x8002, 0x0001, 0x7fff);
+#undef ASSERT_COMP
+}
+
+GST_END_TEST;
+
 static Suite *
 rtp_suite (void)
 {
@@ -183,6 +270,7 @@ rtp_suite (void)
   suite_add_tcase (s, tc_chain);
   tcase_add_test (tc_chain, test_rtp_buffer);
   tcase_add_test (tc_chain, test_rtp_buffer_set_extension_data);
+  tcase_add_test (tc_chain, test_rtp_seqnum_compare);
   return s;
 }
 
