@@ -20,9 +20,6 @@
  * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
  * Boston, MA 02111-1307, USA.
  */
-/* FIXME: remove #if 0 code
- *
- */
 #include <stdlib.h>
 #include <glib.h>
 #include <gtk/gtk.h>
@@ -46,6 +43,7 @@ GST_DEBUG_CATEGORY_STATIC (seek_debug);
 //#define VSINK "aasink"
 //#define VSINK "cacasink"
 
+#define FILL_INTERVAL 100
 //#define UPDATE_INTERVAL 500
 //#define UPDATE_INTERVAL 100
 #define UPDATE_INTERVAL 10
@@ -75,15 +73,21 @@ static const gchar *pipeline_spec;
 static gint64 position = -1;
 static gint64 duration = -1;
 static GtkAdjustment *adjustment;
-static GtkWidget *hscale;
+static GtkWidget *hscale, *statusbar;
+static guint status_id = 0;
 static gboolean stats = FALSE;
 static gboolean elem_seek = FALSE;
 static gboolean verbose = FALSE;
 
+static gboolean is_live = FALSE;
+static gboolean buffering = FALSE;
+static GstBufferingMode mode;
+static gint64 buffering_left;
 static GstState state = GST_STATE_NULL;
 static guint update_id = 0;
 static guint seek_timeout_id = 0;
 static gulong changed_id;
+static guint fill_id = 0;
 
 static gint n_video = 0, n_audio = 0, n_text = 0;
 static gboolean need_streams = TRUE;
@@ -181,10 +185,11 @@ make_mod_pipeline (const gchar * location)
   gst_element_link (src, decoder);
   gst_element_link (decoder, audiosink);
 
-  seekable = gst_element_get_pad (decoder, "src");
+  seekable = gst_element_get_static_pad (decoder, "src");
   seekable_pads = g_list_prepend (seekable_pads, seekable);
   rate_pads = g_list_prepend (rate_pads, seekable);
-  rate_pads = g_list_prepend (rate_pads, gst_element_get_pad (decoder, "sink"));
+  rate_pads =
+      g_list_prepend (rate_pads, gst_element_get_static_pad (decoder, "sink"));
 
   return pipeline;
 }
@@ -222,12 +227,12 @@ make_dv_pipeline (const gchar * location)
   gst_element_link (v_queue, decoder);
   gst_element_link (decoder, videosink);
 
-  setup_dynamic_link (demux, "video", gst_element_get_pad (v_queue, "sink"),
-      NULL);
-  setup_dynamic_link (demux, "audio", gst_element_get_pad (a_queue, "sink"),
-      NULL);
+  setup_dynamic_link (demux, "video", gst_element_get_static_pad (v_queue,
+          "sink"), NULL);
+  setup_dynamic_link (demux, "audio", gst_element_get_static_pad (a_queue,
+          "sink"), NULL);
 
-  seekable = gst_element_get_pad (decoder, "src");
+  seekable = gst_element_get_static_pad (decoder, "src");
   seekable_pads = g_list_prepend (seekable_pads, seekable);
   rate_pads = g_list_prepend (rate_pads, seekable);
 
@@ -254,8 +259,8 @@ make_wav_pipeline (const gchar * location)
 
   gst_element_link (src, decoder);
 
-  setup_dynamic_link (decoder, "src", gst_element_get_pad (audiosink, "sink"),
-      NULL);
+  setup_dynamic_link (decoder, "src", gst_element_get_static_pad (audiosink,
+          "sink"), NULL);
 
   seekable_elements = g_list_prepend (seekable_elements, audiosink);
 
@@ -288,10 +293,11 @@ make_flac_pipeline (const gchar * location)
   gst_element_link (src, decoder);
   gst_element_link (decoder, audiosink);
 
-  seekable = gst_element_get_pad (decoder, "src");
+  seekable = gst_element_get_static_pad (decoder, "src");
   seekable_pads = g_list_prepend (seekable_pads, seekable);
   rate_pads = g_list_prepend (rate_pads, seekable);
-  rate_pads = g_list_prepend (rate_pads, gst_element_get_pad (decoder, "sink"));
+  rate_pads =
+      g_list_prepend (rate_pads, gst_element_get_static_pad (decoder, "sink"));
 
   return pipeline;
 }
@@ -319,10 +325,11 @@ make_sid_pipeline (const gchar * location)
   gst_element_link (src, decoder);
   gst_element_link (decoder, audiosink);
 
-  seekable = gst_element_get_pad (decoder, "src");
+  seekable = gst_element_get_static_pad (decoder, "src");
   seekable_pads = g_list_prepend (seekable_pads, seekable);
   rate_pads = g_list_prepend (rate_pads, seekable);
-  rate_pads = g_list_prepend (rate_pads, gst_element_get_pad (decoder, "sink"));
+  rate_pads =
+      g_list_prepend (rate_pads, gst_element_get_static_pad (decoder, "sink"));
 
   return pipeline;
 }
@@ -351,10 +358,11 @@ make_parse_pipeline (const gchar * location)
   gst_element_link (src, parser);
   gst_element_link (parser, fakesink);
 
-  seekable = gst_element_get_pad (parser, "src");
+  seekable = gst_element_get_static_pad (parser, "src");
   seekable_pads = g_list_prepend (seekable_pads, seekable);
   rate_pads = g_list_prepend (rate_pads, seekable);
-  rate_pads = g_list_prepend (rate_pads, gst_element_get_pad (parser, "sink"));
+  rate_pads =
+      g_list_prepend (rate_pads, gst_element_get_static_pad (parser, "sink"));
 
   return pipeline;
 }
@@ -390,17 +398,18 @@ make_vorbis_pipeline (const gchar * location)
   gst_element_link (decoder, convert);
   gst_element_link (convert, audiosink);
 
-  pad = gst_element_get_pad (decoder, "sink");
+  pad = gst_element_get_static_pad (decoder, "sink");
   gst_element_add_pad (audio_bin, gst_ghost_pad_new ("sink", pad));
   gst_object_unref (pad);
 
-  setup_dynamic_link (demux, NULL, gst_element_get_pad (audio_bin, "sink"),
-      NULL);
+  setup_dynamic_link (demux, NULL, gst_element_get_static_pad (audio_bin,
+          "sink"), NULL);
 
-  seekable = gst_element_get_pad (decoder, "src");
+  seekable = gst_element_get_static_pad (decoder, "src");
   seekable_pads = g_list_prepend (seekable_pads, seekable);
   rate_pads = g_list_prepend (rate_pads, seekable);
-  rate_pads = g_list_prepend (rate_pads, gst_element_get_pad (decoder, "sink"));
+  rate_pads =
+      g_list_prepend (rate_pads, gst_element_get_static_pad (decoder, "sink"));
 
   return pipeline;
 }
@@ -435,17 +444,18 @@ make_theora_pipeline (const gchar * location)
   gst_element_link (decoder, convert);
   gst_element_link (convert, videosink);
 
-  pad = gst_element_get_pad (decoder, "sink");
+  pad = gst_element_get_static_pad (decoder, "sink");
   gst_element_add_pad (video_bin, gst_ghost_pad_new ("sink", pad));
   gst_object_unref (pad);
 
-  setup_dynamic_link (demux, NULL, gst_element_get_pad (video_bin, "sink"),
-      NULL);
+  setup_dynamic_link (demux, NULL, gst_element_get_static_pad (video_bin,
+          "sink"), NULL);
 
-  seekable = gst_element_get_pad (decoder, "src");
+  seekable = gst_element_get_static_pad (decoder, "src");
   seekable_pads = g_list_prepend (seekable_pads, seekable);
   rate_pads = g_list_prepend (rate_pads, seekable);
-  rate_pads = g_list_prepend (rate_pads, gst_element_get_pad (decoder, "sink"));
+  rate_pads =
+      g_list_prepend (rate_pads, gst_element_get_static_pad (decoder, "sink"));
 
   return pipeline;
 }
@@ -488,12 +498,12 @@ make_vorbis_theora_pipeline (const gchar * location)
   gst_element_link (a_decoder, a_convert);
   gst_element_link (a_convert, audiosink);
 
-  pad = gst_element_get_pad (a_queue, "sink");
+  pad = gst_element_get_static_pad (a_queue, "sink");
   gst_element_add_pad (audio_bin, gst_ghost_pad_new ("sink", pad));
   gst_object_unref (pad);
 
-  setup_dynamic_link (demux, NULL, gst_element_get_pad (audio_bin, "sink"),
-      NULL);
+  setup_dynamic_link (demux, NULL, gst_element_get_static_pad (audio_bin,
+          "sink"), NULL);
 
   video_bin = gst_bin_new ("v_decoder_bin");
   v_queue = gst_element_factory_make_or_warn ("queue", "v_queue");
@@ -514,18 +524,19 @@ make_vorbis_theora_pipeline (const gchar * location)
   gst_element_link_many (v_queue, v_decoder, v_convert, v_scale, videosink,
       NULL);
 
-  pad = gst_element_get_pad (v_queue, "sink");
+  pad = gst_element_get_static_pad (v_queue, "sink");
   gst_element_add_pad (video_bin, gst_ghost_pad_new ("sink", pad));
   gst_object_unref (pad);
 
-  setup_dynamic_link (demux, NULL, gst_element_get_pad (video_bin, "sink"),
-      NULL);
+  setup_dynamic_link (demux, NULL, gst_element_get_static_pad (video_bin,
+          "sink"), NULL);
 
-  seekable = gst_element_get_pad (a_decoder, "src");
+  seekable = gst_element_get_static_pad (a_decoder, "src");
   seekable_pads = g_list_prepend (seekable_pads, seekable);
   rate_pads = g_list_prepend (rate_pads, seekable);
   rate_pads =
-      g_list_prepend (rate_pads, gst_element_get_pad (a_decoder, "sink"));
+      g_list_prepend (rate_pads, gst_element_get_static_pad (a_decoder,
+          "sink"));
 
   return pipeline;
 }
@@ -567,12 +578,12 @@ make_avi_msmpeg4v3_mp3_pipeline (const gchar * location)
 
   gst_bin_add (GST_BIN (pipeline), audio_bin);
 
-  pad = gst_element_get_pad (a_queue, "sink");
+  pad = gst_element_get_static_pad (a_queue, "sink");
   gst_element_add_pad (audio_bin, gst_ghost_pad_new ("sink", pad));
   gst_object_unref (pad);
 
-  setup_dynamic_link (demux, NULL, gst_element_get_pad (audio_bin, "sink"),
-      NULL);
+  setup_dynamic_link (demux, NULL, gst_element_get_static_pad (audio_bin,
+          "sink"), NULL);
 
   video_bin = gst_bin_new ("v_decoder_bin");
   v_queue = gst_element_factory_make_or_warn ("queue", "v_queue");
@@ -590,18 +601,19 @@ make_avi_msmpeg4v3_mp3_pipeline (const gchar * location)
 
   gst_bin_add (GST_BIN (pipeline), video_bin);
 
-  pad = gst_element_get_pad (v_queue, "sink");
+  pad = gst_element_get_static_pad (v_queue, "sink");
   gst_element_add_pad (video_bin, gst_ghost_pad_new ("sink", pad));
   gst_object_unref (pad);
 
-  setup_dynamic_link (demux, NULL, gst_element_get_pad (video_bin, "sink"),
-      NULL);
+  setup_dynamic_link (demux, NULL, gst_element_get_static_pad (video_bin,
+          "sink"), NULL);
 
-  seekable = gst_element_get_pad (a_decoder, "src");
+  seekable = gst_element_get_static_pad (a_decoder, "src");
   seekable_pads = g_list_prepend (seekable_pads, seekable);
   rate_pads = g_list_prepend (rate_pads, seekable);
   rate_pads =
-      g_list_prepend (rate_pads, gst_element_get_pad (a_decoder, "sink"));
+      g_list_prepend (rate_pads, gst_element_get_static_pad (a_decoder,
+          "sink"));
 
   return pipeline;
 }
@@ -634,10 +646,11 @@ make_mp3_pipeline (const gchar * location)
   gst_element_link (decoder, queue);
   gst_element_link (queue, osssink);
 
-  seekable = gst_element_get_pad (queue, "src");
+  seekable = gst_element_get_static_pad (queue, "src");
   seekable_pads = g_list_prepend (seekable_pads, seekable);
   rate_pads = g_list_prepend (rate_pads, seekable);
-  rate_pads = g_list_prepend (rate_pads, gst_element_get_pad (decoder, "sink"));
+  rate_pads =
+      g_list_prepend (rate_pads, gst_element_get_static_pad (decoder, "sink"));
 
   return pipeline;
 }
@@ -673,14 +686,15 @@ make_avi_pipeline (const gchar * location)
   gst_bin_add (GST_BIN (audio_bin), audiosink);
   gst_element_set_state (audio_bin, GST_STATE_PAUSED);
 
-  setup_dynamic_link (demux, "audio_00", gst_element_get_pad (a_decoder,
+  setup_dynamic_link (demux, "audio_00", gst_element_get_static_pad (a_decoder,
           "sink"), audio_bin);
 
-  seekable = gst_element_get_pad (a_queue, "src");
+  seekable = gst_element_get_static_pad (a_queue, "src");
   seekable_pads = g_list_prepend (seekable_pads, seekable);
   rate_pads = g_list_prepend (rate_pads, seekable);
   rate_pads =
-      g_list_prepend (rate_pads, gst_element_get_pad (a_decoder, "sink"));
+      g_list_prepend (rate_pads, gst_element_get_static_pad (a_decoder,
+          "sink"));
 
   video_bin = gst_bin_new ("v_decoder_bin");
   v_decoder = gst_element_factory_make_or_warn ("ffmpegdecall", "v_dec");
@@ -694,14 +708,15 @@ make_avi_pipeline (const gchar * location)
 
   gst_element_set_state (video_bin, GST_STATE_PAUSED);
 
-  setup_dynamic_link (demux, "video_00", gst_element_get_pad (v_decoder,
+  setup_dynamic_link (demux, "video_00", gst_element_get_static_pad (v_decoder,
           "sink"), video_bin);
 
-  seekable = gst_element_get_pad (v_queue, "src");
+  seekable = gst_element_get_static_pad (v_queue, "src");
   seekable_pads = g_list_prepend (seekable_pads, seekable);
   rate_pads = g_list_prepend (rate_pads, seekable);
   rate_pads =
-      g_list_prepend (rate_pads, gst_element_get_pad (v_decoder, "sink"));
+      g_list_prepend (rate_pads, gst_element_get_static_pad (v_decoder,
+          "sink"));
 
   return pipeline;
 }
@@ -741,11 +756,11 @@ make_mpeg_pipeline (const gchar * location)
 
   gst_bin_add (GST_BIN (pipeline), audio_bin);
 
-  pad = gst_element_get_pad (a_decoder, "sink");
+  pad = gst_element_get_static_pad (a_decoder, "sink");
   gst_element_add_pad (audio_bin, gst_ghost_pad_new ("sink", pad));
   gst_object_unref (pad);
 
-  setup_dynamic_link (demux, "audio_c0", gst_element_get_pad (audio_bin,
+  setup_dynamic_link (demux, "audio_c0", gst_element_get_static_pad (audio_bin,
           "sink"), NULL);
 
   video_bin = gst_bin_new ("v_decoder_bin");
@@ -765,18 +780,19 @@ make_mpeg_pipeline (const gchar * location)
 
   gst_bin_add (GST_BIN (pipeline), video_bin);
 
-  pad = gst_element_get_pad (v_decoder, "sink");
+  pad = gst_element_get_static_pad (v_decoder, "sink");
   gst_element_add_pad (video_bin, gst_ghost_pad_new ("sink", pad));
   gst_object_unref (pad);
 
-  setup_dynamic_link (demux, "video_e0", gst_element_get_pad (video_bin,
+  setup_dynamic_link (demux, "video_e0", gst_element_get_static_pad (video_bin,
           "sink"), NULL);
 
-  seekable = gst_element_get_pad (v_filter, "src");
+  seekable = gst_element_get_static_pad (v_filter, "src");
   seekable_pads = g_list_prepend (seekable_pads, seekable);
   rate_pads = g_list_prepend (rate_pads, seekable);
   rate_pads =
-      g_list_prepend (rate_pads, gst_element_get_pad (v_decoder, "sink"));
+      g_list_prepend (rate_pads, gst_element_get_static_pad (v_decoder,
+          "sink"));
 
   return pipeline;
 }
@@ -816,14 +832,15 @@ make_mpegnt_pipeline (const gchar * location)
   gst_bin_add (GST_BIN (audio_bin), a_queue);
   gst_bin_add (GST_BIN (audio_bin), audiosink);
 
-  setup_dynamic_link (demux, "audio_00", gst_element_get_pad (a_decoder,
+  setup_dynamic_link (demux, "audio_00", gst_element_get_static_pad (a_decoder,
           "sink"), audio_bin);
 
-  seekable = gst_element_get_pad (a_queue, "src");
+  seekable = gst_element_get_static_pad (a_queue, "src");
   seekable_pads = g_list_prepend (seekable_pads, seekable);
   rate_pads = g_list_prepend (rate_pads, seekable);
   rate_pads =
-      g_list_prepend (rate_pads, gst_element_get_pad (a_decoder, "sink"));
+      g_list_prepend (rate_pads, gst_element_get_static_pad (a_decoder,
+          "sink"));
 
   video_bin = gst_bin_new ("v_decoder_bin");
   v_decoder = gst_element_factory_make_or_warn ("mpeg2dec", "v_dec");
@@ -833,14 +850,15 @@ make_mpegnt_pipeline (const gchar * location)
 
   gst_bin_add_many (GST_BIN (video_bin), v_decoder, v_filter, videosink, NULL);
 
-  setup_dynamic_link (demux, "video_00", gst_element_get_pad (v_decoder,
+  setup_dynamic_link (demux, "video_00", gst_element_get_static_pad (v_decoder,
           "sink"), video_bin);
 
-  seekable = gst_element_get_pad (v_decoder, "src");
+  seekable = gst_element_get_static_pad (v_decoder, "src");
   seekable_pads = g_list_prepend (seekable_pads, seekable);
   rate_pads = g_list_prepend (rate_pads, seekable);
   rate_pads =
-      g_list_prepend (rate_pads, gst_element_get_pad (v_decoder, "sink"));
+      g_list_prepend (rate_pads, gst_element_get_static_pad (v_decoder,
+          "sink"));
 
   return pipeline;
 }
@@ -886,7 +904,7 @@ static GstElement *
 make_parselaunch_pipeline (const gchar * description)
 {
   GstElement *pipeline;
-  GError *error;
+  GError *error = NULL;
 
   pipeline = gst_parse_launch (description, &error);
 
@@ -1082,6 +1100,55 @@ set_scale (gdouble value)
   gtk_widget_queue_draw (hscale);
 }
 
+static gboolean
+update_fill (gpointer data)
+{
+  if (elem_seek) {
+    if (seekable_elements) {
+      GstElement *element = GST_ELEMENT (seekable_elements->data);
+      GstQuery *query;
+
+      query = gst_query_new_buffering (GST_FORMAT_PERCENT);
+      if (gst_element_query (element, query)) {
+        gint64 start, stop;
+        GstFormat format;
+        gdouble fill;
+        gboolean busy;
+        gint percent;
+
+        gst_query_parse_buffering_percent (query, &busy, &percent);
+
+        if (buffering && !busy) {
+          /* if we were buffering but not anymore, start playing */
+          if (state == GST_STATE_PLAYING && !is_live) {
+            fprintf (stderr, "setting pipeline to PLAYING ...\n");
+            gst_element_set_state (pipeline, GST_STATE_PLAYING);
+            gtk_statusbar_pop (GTK_STATUSBAR (statusbar), status_id);
+            gtk_statusbar_push (GTK_STATUSBAR (statusbar), status_id,
+                "Playing");
+          }
+          state = GST_STATE_PAUSED;
+          buffering = FALSE;
+        }
+
+        gst_query_parse_buffering_range (query, &format, &start, &stop, NULL);
+
+        GST_DEBUG ("start %" G_GINT64_FORMAT ", stop %" G_GINT64_FORMAT,
+            start, stop);
+
+        if (stop != -1)
+          fill = 100.0 * stop / GST_FORMAT_PERCENT_MAX;
+        else
+          fill = 100.0;
+
+        gtk_range_set_fill_level (GTK_RANGE (hscale), fill);
+      }
+      gst_query_unref (query);
+    }
+  }
+  return TRUE;
+}
+
 static gboolean
 update_scale (gpointer data)
 {
@@ -1127,6 +1194,7 @@ update_scale (gpointer data)
 static void do_seek (GtkWidget * widget);
 static void connect_bus_signals (GstElement * pipeline);
 static void set_update_scale (gboolean active);
+static void set_update_fill (gboolean active);
 
 static gboolean
 end_scrub (GtkWidget * widget)
@@ -1244,6 +1312,24 @@ seek_cb (GtkWidget * widget)
   }
 }
 
+static void
+set_update_fill (gboolean active)
+{
+  GST_DEBUG ("fill scale is %d", active);
+
+  if (active) {
+    if (fill_id == 0) {
+      fill_id =
+          g_timeout_add (FILL_INTERVAL, (GtkFunction) update_fill, pipeline);
+    }
+  } else {
+    if (fill_id) {
+      g_source_remove (fill_id);
+      fill_id = 0;
+    }
+  }
+}
+
 static void
 set_update_scale (gboolean active)
 {
@@ -1323,18 +1409,27 @@ play_cb (GtkButton * button, gpointer data)
 
   if (state != GST_STATE_PLAYING) {
     g_print ("PLAY pipeline\n");
-    ret = gst_element_set_state (pipeline, GST_STATE_PLAYING);
-    if (ret == GST_STATE_CHANGE_FAILURE)
-      goto failed;
-    //do_seek(hscale);
+    gtk_statusbar_pop (GTK_STATUSBAR (statusbar), status_id);
 
+    ret = gst_element_set_state (pipeline, GST_STATE_PLAYING);
+    switch (ret) {
+      case GST_STATE_CHANGE_FAILURE:
+        goto failed;
+      case GST_STATE_CHANGE_NO_PREROLL:
+        is_live = TRUE;
+        break;
+      default:
+        break;
+    }
     state = GST_STATE_PLAYING;
+    gtk_statusbar_push (GTK_STATUSBAR (statusbar), status_id, "Playing");
   }
   return;
 
 failed:
   {
     g_print ("PLAY failed\n");
+    gtk_statusbar_push (GTK_STATUSBAR (statusbar), status_id, "Play failed");
   }
 }
 
@@ -1344,18 +1439,28 @@ pause_cb (GtkButton * button, gpointer data)
   if (state != GST_STATE_PAUSED) {
     GstStateChangeReturn ret;
 
+    gtk_statusbar_pop (GTK_STATUSBAR (statusbar), status_id);
     g_print ("PAUSE pipeline\n");
     ret = gst_element_set_state (pipeline, GST_STATE_PAUSED);
-    if (ret == GST_STATE_CHANGE_FAILURE)
-      goto failed;
+    switch (ret) {
+      case GST_STATE_CHANGE_FAILURE:
+        goto failed;
+      case GST_STATE_CHANGE_NO_PREROLL:
+        is_live = TRUE;
+        break;
+      default:
+        break;
+    }
 
     state = GST_STATE_PAUSED;
+    gtk_statusbar_push (GTK_STATUSBAR (statusbar), status_id, "Paused");
   }
   return;
 
 failed:
   {
     g_print ("PAUSE failed\n");
+    gtk_statusbar_push (GTK_STATUSBAR (statusbar), status_id, "Pause failed");
   }
 }
 
@@ -1366,14 +1471,20 @@ stop_cb (GtkButton * button, gpointer data)
     GstStateChangeReturn ret;
 
     g_print ("READY pipeline\n");
+    gtk_statusbar_pop (GTK_STATUSBAR (statusbar), status_id);
+
     ret = gst_element_set_state (pipeline, GST_STATE_READY);
     if (ret == GST_STATE_CHANGE_FAILURE)
       goto failed;
 
     state = GST_STATE_READY;
+    gtk_statusbar_push (GTK_STATUSBAR (statusbar), status_id, "Stopped");
 
+    is_live = FALSE;
+    buffering = FALSE;
     set_update_scale (FALSE);
     set_scale (0.0);
+    set_update_fill (FALSE);
 
     if (pipeline_type == 16)
       clear_streams (pipeline);
@@ -1396,6 +1507,7 @@ stop_cb (GtkButton * button, gpointer data)
 failed:
   {
     g_print ("STOP failed\n");
+    gtk_statusbar_push (GTK_STATUSBAR (statusbar), status_id, "Stop failed");
   }
 }
 
@@ -1505,19 +1617,31 @@ vis_toggle_cb (GtkToggleButton * button, GstPipeline * pipeline)
 static void
 audio_toggle_cb (GtkToggleButton * button, GstPipeline * pipeline)
 {
-  update_flag (pipeline, 1, gtk_toggle_button_get_active (button));
+  gboolean state;
+
+  state = gtk_toggle_button_get_active (button);
+  update_flag (pipeline, 1, state);
+  gtk_widget_set_sensitive (audio_combo, state);
 }
 
 static void
 video_toggle_cb (GtkToggleButton * button, GstPipeline * pipeline)
 {
-  update_flag (pipeline, 0, gtk_toggle_button_get_active (button));
+  gboolean state;
+
+  state = gtk_toggle_button_get_active (button);
+  update_flag (pipeline, 0, state);
+  gtk_widget_set_sensitive (video_combo, state);
 }
 
 static void
 text_toggle_cb (GtkToggleButton * button, GstPipeline * pipeline)
 {
-  update_flag (pipeline, 2, gtk_toggle_button_get_active (button));
+  gboolean state;
+
+  state = gtk_toggle_button_get_active (button);
+  update_flag (pipeline, 2, state);
+  gtk_widget_set_sensitive (text_combo, state);
 }
 
 static void
@@ -1559,6 +1683,7 @@ update_streams (GstPipeline * pipeline)
     GstTagList *tags;
     gchar *name;
     gint active_idx;
+    gboolean state;
 
     /* remove previous info */
     clear_streams (GST_ELEMENT_CAST (pipeline));
@@ -1578,7 +1703,8 @@ update_streams (GstPipeline * pipeline)
       gtk_combo_box_append_text (GTK_COMBO_BOX (video_combo), name);
       g_free (name);
     }
-    gtk_widget_set_sensitive (video_combo, n_video > 0);
+    state = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (video_checkbox));
+    gtk_widget_set_sensitive (video_combo, state && n_video > 0);
     gtk_combo_box_set_active (GTK_COMBO_BOX (video_combo), active_idx);
 
     active_idx = 0;
@@ -1589,18 +1715,32 @@ update_streams (GstPipeline * pipeline)
       gtk_combo_box_append_text (GTK_COMBO_BOX (audio_combo), name);
       g_free (name);
     }
-    gtk_widget_set_sensitive (audio_combo, n_audio > 0);
+    state = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (audio_checkbox));
+    gtk_widget_set_sensitive (audio_combo, state && n_audio > 0);
     gtk_combo_box_set_active (GTK_COMBO_BOX (audio_combo), active_idx);
 
     active_idx = 0;
     for (i = 0; i < n_text; i++) {
       g_signal_emit_by_name (pipeline, "get-text-tags", i, &tags);
-      /* find good name for the label */
-      name = g_strdup_printf ("text %d", i + 1);
+      name = NULL;
+      if (tags) {
+        const GValue *value;
+
+        /* get the language code if we can */
+        value = gst_tag_list_get_value_index (tags, GST_TAG_LANGUAGE_CODE, 0);
+        if (value && G_VALUE_HOLDS_STRING (value)) {
+          name = g_strdup_printf ("text %s", g_value_get_string (value));
+        }
+      }
+      /* find good name for the label if we didn't use a tag */
+      if (name == NULL)
+        name = g_strdup_printf ("text %d", i + 1);
+
       gtk_combo_box_append_text (GTK_COMBO_BOX (text_combo), name);
       g_free (name);
     }
-    gtk_widget_set_sensitive (text_combo, n_text > 0);
+    state = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (text_checkbox));
+    gtk_widget_set_sensitive (text_combo, state && n_text > 0);
     gtk_combo_box_set_active (GTK_COMBO_BOX (text_combo), active_idx);
 
     need_streams = FALSE;
@@ -1841,6 +1981,89 @@ msg_segment_done (GstBus * bus, GstMessage * message, GstPipeline * pipeline)
     g_print ("segment seek failed\n");
 }
 
+/* in stream buffering mode we PAUSE the pipeline until we receive a 100%
+ * message */
+static void
+do_stream_buffering (gint percent)
+{
+  gchar *bufstr;
+
+  gtk_statusbar_pop (GTK_STATUSBAR (statusbar), status_id);
+  bufstr = g_strdup_printf ("Buffering...%d", percent);
+  gtk_statusbar_push (GTK_STATUSBAR (statusbar), status_id, bufstr);
+  g_free (bufstr);
+
+  if (percent == 100) {
+    /* a 100% message means buffering is done */
+    buffering = FALSE;
+    /* if the desired state is playing, go back */
+    if (state == GST_STATE_PLAYING) {
+      /* no state management needed for live pipelines */
+      if (!is_live) {
+        fprintf (stderr, "Done buffering, setting pipeline to PLAYING ...\n");
+        gst_element_set_state (pipeline, GST_STATE_PLAYING);
+      }
+      gtk_statusbar_pop (GTK_STATUSBAR (statusbar), status_id);
+      gtk_statusbar_push (GTK_STATUSBAR (statusbar), status_id, "Playing");
+    }
+  } else {
+    /* buffering busy */
+    if (buffering == FALSE && state == GST_STATE_PLAYING) {
+      /* we were not buffering but PLAYING, PAUSE  the pipeline. */
+      if (!is_live) {
+        fprintf (stderr, "Buffering, setting pipeline to PAUSED ...\n");
+        gst_element_set_state (pipeline, GST_STATE_PAUSED);
+      }
+    }
+    buffering = TRUE;
+  }
+}
+
+static void
+do_download_buffering (gint percent)
+{
+  if (!buffering && percent < 100) {
+    gchar *bufstr;
+
+    buffering = TRUE;
+
+    bufstr = g_strdup_printf ("Downloading...");
+    gtk_statusbar_push (GTK_STATUSBAR (statusbar), status_id, bufstr);
+    g_free (bufstr);
+
+    /* once we get a buffering message, we'll do the fill update */
+    set_update_fill (TRUE);
+
+    if (state == GST_STATE_PLAYING && !is_live) {
+      fprintf (stderr, "Downloading, setting pipeline to PAUSED ...\n");
+      gst_element_set_state (pipeline, GST_STATE_PAUSED);
+    }
+  }
+}
+
+static void
+msg_buffering (GstBus * bus, GstMessage * message, GstPipeline * data)
+{
+  gint percent;
+
+  gst_message_parse_buffering (message, &percent);
+
+  /* get more stats */
+  gst_message_parse_buffering_stats (message, &mode, NULL, NULL,
+      &buffering_left);
+
+  switch (mode) {
+    case GST_BUFFERING_DOWNLOAD:
+      do_download_buffering (percent);
+      break;
+    case GST_BUFFERING_LIVE:
+    case GST_BUFFERING_TIMESHIFT:
+    case GST_BUFFERING_STREAM:
+      do_stream_buffering (percent);
+      break;
+  }
+}
+
 static void
 connect_bus_signals (GstElement * pipeline)
 {
@@ -1869,6 +2092,8 @@ connect_bus_signals (GstElement * pipeline)
       pipeline);
   g_signal_connect (bus, "message::segment-done", (GCallback) message_received,
       pipeline);
+  g_signal_connect (bus, "message::buffering", (GCallback) msg_buffering,
+      pipeline);
 
   gst_object_unref (bus);
 }
@@ -1943,6 +2168,9 @@ main (int argc, char **argv)
   /* initialize gui elements ... */
   tips = gtk_tooltips_new ();
   window = gtk_window_new (GTK_WINDOW_TOPLEVEL);
+  statusbar = gtk_statusbar_new ();
+  status_id = gtk_statusbar_get_context_id (GTK_STATUSBAR (statusbar), "seek");
+  gtk_statusbar_push (GTK_STATUSBAR (statusbar), status_id, "Stopped");
   hbox = gtk_hbox_new (FALSE, 0);
   vbox = gtk_vbox_new (FALSE, 0);
   flagtable = gtk_table_new (4, 2, FALSE);
@@ -2030,10 +2258,10 @@ main (int argc, char **argv)
     mute_checkbox = gtk_check_button_new_with_label ("Mute");
     volume_spinbutton = gtk_spin_button_new_with_range (0, 10.0, 0.1);
     gtk_spin_button_set_value (GTK_SPIN_BUTTON (volume_spinbutton), 1.0);
-    gtk_box_pack_start (GTK_BOX (boxes), vis_checkbox, TRUE, TRUE, 2);
-    gtk_box_pack_start (GTK_BOX (boxes), audio_checkbox, TRUE, TRUE, 2);
     gtk_box_pack_start (GTK_BOX (boxes), video_checkbox, TRUE, TRUE, 2);
+    gtk_box_pack_start (GTK_BOX (boxes), audio_checkbox, TRUE, TRUE, 2);
     gtk_box_pack_start (GTK_BOX (boxes), text_checkbox, TRUE, TRUE, 2);
+    gtk_box_pack_start (GTK_BOX (boxes), vis_checkbox, TRUE, TRUE, 2);
     gtk_box_pack_start (GTK_BOX (boxes), mute_checkbox, TRUE, TRUE, 2);
     gtk_box_pack_start (GTK_BOX (boxes), volume_spinbutton, TRUE, TRUE, 2);
     gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (vis_checkbox), FALSE);
@@ -2098,6 +2326,7 @@ main (int argc, char **argv)
     gtk_box_pack_start (GTK_BOX (vbox), boxes2, TRUE, TRUE, 2);
   }
   gtk_box_pack_start (GTK_BOX (vbox), hscale, TRUE, TRUE, 2);
+  gtk_box_pack_start (GTK_BOX (vbox), statusbar, TRUE, TRUE, 2);
 
   /* connect things ... */
   g_signal_connect (G_OBJECT (play_button), "clicked", G_CALLBACK (play_cb),
